@@ -1,11 +1,11 @@
 /*
  * radius.c	Functions to send/receive radius packets.
  *
- * Version:	$Id: radius.c,v 1.79 2002/07/26 14:43:21 aland Exp $
+ * Version:	$Id: radius.c,v 1.86 2002/10/25 16:30:56 aland Exp $
  *
  */
 
-static const char rcsid[] = "$Id: radius.c,v 1.79 2002/07/26 14:43:21 aland Exp $";
+static const char rcsid[] = "$Id: radius.c,v 1.86 2002/10/25 16:30:56 aland Exp $";
 
 #include	"autoconf.h"
 #include	"md5.h"
@@ -278,36 +278,45 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 			  case PW_TYPE_STRING:
 			  case PW_TYPE_OCTETS:
 				  /*
-				   *  Hmm... this is based on names
-				   *  right now.  We really shouldn't do
-				   *  this.  It should be based on the value
-				   *  of the attribute (VSA or not).
+				   *  FIXME: HACK for non-updated dictionaries.
+				   *  REMOVE in a future release.
 				   */
 				  if ((strcmp(reply->name, "Ascend-Send-Secret") == 0) ||
 				      (strcmp(reply->name, "Ascend-Receive-Secret") == 0)) {
+					  reply->flags.encrypt = FLAG_ENCRYPT_ASCEND_SECRET;
+				  }
+				  if (reply->attribute == PW_USER_PASSWORD) {
+					  reply->flags.encrypt = FLAG_ENCRYPT_USER_PASSWORD;
+				  }
+
+				  /*
+				   *  Encrypt the various password styles
+				   */
+				  switch (reply->flags.encrypt) {
+				  default:
+					  break;
+
+				  case FLAG_ENCRYPT_USER_PASSWORD:
+				    rad_pwencode((char *)reply->strvalue,
+						 &(reply->length),
+						 (char *)secret,
+						 (char *)packet->vector);
+				    break;
+
+				  case FLAG_ENCRYPT_TUNNEL_PASSWORD:
+					  rad_tunnel_pwencode(reply->strvalue,
+							      &(reply->length),
+							      secret,
+							      packet->vector);
+					  break;
+
+
+				  case FLAG_ENCRYPT_ASCEND_SECRET:
 					  make_secret(digest, packet->vector,
 						      secret, reply->strvalue);
 					  memcpy(reply->strvalue, digest, AUTH_VECTOR_LEN );
 					  reply->length = AUTH_VECTOR_LEN;
-				  }
-
-				  /* 
-				   *    Encrypt Tunnel-Password
-				   *    here -- cparker@starnetusa.net
-				   */
-
-				  /*
-				   *	Encrypt with Tunnel-Password style if needed.
-				   */
-				  if (reply->flags.encrypt == 2) {
-
-				    rad_tunnel_pwencode(reply->strvalue,
-							&(reply->length),
-							secret,
-							packet->vector);
-				    
-				  }
-			  
+				  } /* switch over encryption flags */
 
 				  len = reply->length;
 
@@ -323,7 +332,7 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 						  len++;
 						  *ptr++ = reply->flags.tag;
 
-					  } else if (reply->flags.encrypt == 2) {
+					  } else if (reply->flags.encrypt == FLAG_ENCRYPT_TUNNEL_PASSWORD) {
 						  /*
 						   *  Tunnel passwords
 						   *  REQUIRE a tag,
@@ -698,7 +707,7 @@ RADIUS_PACKET *rad_recv(int fd)
 	 *	"The minimum length is 20 ..."
 	 */
 	if (packet->data_len < AUTH_HDR_LEN) {
-		librad_log("WARNING: Malformed RADIUS packet from host %s: too short (%d < %d)",
+		librad_log("WARNING: Malformed RADIUS packet from host %s: too short (received %d < minimum %d)",
 			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
 			   packet->data_len, AUTH_HDR_LEN);
 		free(packet);
@@ -711,7 +720,7 @@ RADIUS_PACKET *rad_recv(int fd)
 	 *	" ... and maximum length is 4096."
 	 */
 	if (packet->data_len > MAX_PACKET_LEN) {
-		librad_log("WARNING: Malformed RADIUS packet from host %s: too long (%d > %d)",
+		librad_log("WARNING: Malformed RADIUS packet from host %s: too long (received %d > maximum %d)",
 			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
 			   packet->data_len, MAX_PACKET_LEN);
 		free(packet);
@@ -727,6 +736,54 @@ RADIUS_PACKET *rad_recv(int fd)
 	memcpy(&len, hdr->length, sizeof(u_short));
 	totallen = ntohs(len);
 
+	/*
+	 *	Code of 0 is not understood.
+	 *	Code of 16 or greate is not understood.
+	 */
+	if ((hdr->code == 0) ||
+	    (hdr->code >= 16)) {
+		librad_log("WARNING: Bad RADIUS packet from host %s: unknown packet code %d",
+			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
+			   hdr->code);
+		free(packet);
+		return NULL;
+		
+	}
+
+	/*
+	 *	Repeat the length checks.  This time, instead of
+	 *	looking at the data we received, look at the value
+	 *	of the 'length' field inside of the packet.
+	 *
+	 *	Check for packets smaller than the packet header.
+	 *
+	 *	RFC 2865, Section 3., subsection 'length' says:
+	 *
+	 *	"The minimum length is 20 ..."
+	 */
+	if (totallen < AUTH_HDR_LEN) {
+		librad_log("WARNING: Malformed RADIUS packet from host %s: too short (length %d < minimum %d)",
+			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
+			   totallen, AUTH_HDR_LEN);
+		free(packet);
+		return NULL;
+	}
+
+	/*
+	 *	And again, for the value of the 'length' field.
+	 *
+	 *	RFC 2865, Section 3., subsection 'length' says:
+	 *
+	 *	" ... and maximum length is 4096."
+	 */
+	if (totallen > MAX_PACKET_LEN) {
+		librad_log("WARNING: Malformed RADIUS packet from host %s: too long (length %d > maximum %d)",
+			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
+			   totallen, MAX_PACKET_LEN);
+		free(packet);
+		return NULL;
+	}
+
 	/*
 	 *	RFC 2865, Section 3., subsection 'length' says:
 	 *
@@ -736,7 +793,7 @@ RADIUS_PACKET *rad_recv(int fd)
 	 *	i.e. No response to the NAS.
 	 */
 	if (packet->data_len < totallen) {
-		librad_log("WARNING: Malformed RADIUS packet from host %s: received %d octets, packet size says %d",
+		librad_log("WARNING: Malformed RADIUS packet from host %s: received %d octets, packet length says %d",
 			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
 			   packet->data_len, totallen);
 		free(packet);
@@ -754,9 +811,7 @@ RADIUS_PACKET *rad_recv(int fd)
 		 *	We're shortening the packet below, but just
 		 *	to be paranoid, zero out the extra data.
 		 */
-		memset(packet->data + totallen, 0,
-		       packet->data_len - totallen);
-
+		memset(data + totallen, 0, packet->data_len - totallen);
 		packet->data_len = totallen;
 	}
 
@@ -956,8 +1011,7 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 	DICT_ATTR		*attr;
 	uint32_t		lvalue;
 	uint32_t		vendorcode;
-	VALUE_PAIR		*first_pair;
-	VALUE_PAIR		*prev;
+	VALUE_PAIR		**tail;
 	VALUE_PAIR		*pair;
 	uint8_t			*ptr;
 	int			length;
@@ -1069,8 +1123,8 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 	 */
 	ptr = hdr->data;
 	length = packet->data_len - AUTH_HDR_LEN;
-	first_pair = NULL;
-	prev = NULL;
+	packet->vps = NULL;
+	tail = &packet->vps;
 
 	vendorcode = 0;
 	vendorlen  = 0;
@@ -1126,7 +1180,7 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 		 *	FIXME: should we us paircreate() ?
 		 */
 		if ((pair = malloc(sizeof(VALUE_PAIR))) == NULL) {
-			pairfree(&first_pair);
+			pairfree(&packet->vps);
 			librad_log("out of memory");
 			errno = ENOMEM;
 			return -1;
@@ -1135,7 +1189,7 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 		memset(pair, 0, sizeof(VALUE_PAIR));
 		if ((attr = dict_attrbyvalue(attribute)) == NULL) {
 			snprintf(pair->name, sizeof(pair->name), "Attr-%d", attribute);
-			pair->type = PW_TYPE_STRING;
+			pair->type = PW_TYPE_OCTETS;
 		} else {
 			strcpy(pair->name, attr->name);
 			pair->type = attr->type;
@@ -1143,6 +1197,7 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 		}
 		pair->attribute = attribute;
 		pair->length = attrlen;
+		pair->operator = T_OP_EQ;
 		pair->next = NULL;
 		
 		switch (pair->type) {
@@ -1150,26 +1205,15 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 		case PW_TYPE_OCTETS:
 		case PW_TYPE_ABINARY:
 		case PW_TYPE_STRING:
-			/*
-			 *	Hmm... this is based on names right
-			 *	now.  We really shouldn't do this.
-			 *	It should be based on the value of
-			 *	the attribute (VSA or not).
-			 */
-			if ((strcmp(pair->name, "Ascend-Send-Secret") == 0) ||
-			    (strcmp(pair->name, "Ascend-Receive-Secret") == 0)) {
-				uint8_t my_digest[AUTH_VECTOR_LEN];
-				make_secret( my_digest, original->vector,
-					     secret, ptr);
-				memcpy(pair->strvalue, my_digest, AUTH_VECTOR_LEN );
-				pair->strvalue[AUTH_VECTOR_LEN] = '\0';
-				pair->length = strlen(pair->strvalue);
-			} else if (pair->flags.has_tag &&
-				   pair->type == PW_TYPE_STRING) {
+			if (pair->flags.has_tag &&
+			    pair->type == PW_TYPE_STRING) {
+				int offset = 0;
+
 				if(TAG_VALID(*ptr)) {
-					pair->flags.tag = *ptr++;
+					pair->flags.tag = *ptr;
 					pair->length--;
-				} else if (pair->flags.encrypt == 2) {
+					offset = 1;
+				} else if (pair->flags.encrypt == FLAG_ENCRYPT_TUNNEL_PASSWORD) {
 					/*
 					 * from RFC2868 - 3.5.  Tunnel-Password
 					 * If the value of the Tag field is greater than
@@ -1179,12 +1223,12 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 					 * otherwise, the Tag field SHOULD be ignored.
 					 */
 					pair->flags.tag = 0x00;
-					ptr++;
 					pair->length--;
+					offset = 1;
 				} else {
 				       pair->flags.tag = 0x00;
 				}
-				memcpy(pair->strvalue, ptr, 
+				memcpy(pair->strvalue, ptr + offset,
 				       pair->length);
 			} else {
 				/* attrlen always < MAX_STRING_LEN */
@@ -1192,17 +1236,69 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 			        pair->flags.tag = 0;
 			}
 
-			/* FIXME: Decrypt attributes like Tunnel-Password here -cparker */
-			if(pair->flags.encrypt == 2) {
-
-			        rad_tunnel_pwdecode((char *)pair->strvalue,
-						     &pair->length, 
-						     secret,
-						     (char *)original->vector);
-
+			/*
+			 *  FIXME: HACK for non-updated dictionaries.
+			 *  REMOVE in a future release.
+			 */
+			if ((strcmp(pair->name, "Ascend-Send-Secret") == 0) ||
+			    (strcmp(pair->name, "Ascend-Receive-Secret") == 0)) {
+				pair->flags.encrypt = FLAG_ENCRYPT_ASCEND_SECRET;
+			}
+			if (pair->attribute == PW_USER_PASSWORD) {
+				pair->flags.encrypt = FLAG_ENCRYPT_USER_PASSWORD;
 			}
 
-			break;
+			/*
+			 *	Decrypt passwords here.
+			 */
+			switch (pair->flags.encrypt) {
+			default:
+				break;
+
+				/*
+				 *  User-Password
+				 */
+			case FLAG_ENCRYPT_USER_PASSWORD:
+				rad_pwdecode((char *)pair->strvalue,
+					     pair->length, secret,
+					     (char *)packet->vector);
+				pair->lvalue = 1; /* see main/auth.c */
+
+				/*
+				 *  FIXME: Allow NUL's in the password,
+				 *  too??
+				 */
+				pair->length = strlen(pair->strvalue);
+				break;
+
+				/*
+				 *  Tunnel-Password
+				 */
+			case FLAG_ENCRYPT_TUNNEL_PASSWORD:
+			        rad_tunnel_pwdecode((char *)pair->strvalue,
+						    &pair->length, 
+						    secret,
+						    (char *)original->vector);
+				break;
+
+				/*
+				 *  Ascend-Send-Secret
+				 *  Ascend-Receive-Secret
+				 */
+			case FLAG_ENCRYPT_ASCEND_SECRET:
+				{
+					uint8_t my_digest[AUTH_VECTOR_LEN];
+					make_secret(my_digest,
+						    original->vector,
+						    secret, ptr);
+					memcpy(pair->strvalue, my_digest,
+					       AUTH_VECTOR_LEN );
+					pair->strvalue[AUTH_VECTOR_LEN] = '\0';
+					pair->length = strlen(pair->strvalue);
+				}
+				break;
+			} /* switch over encryption flags */
+			break;	/* from octets/string/abinary */
 			
 		case PW_TYPE_INTEGER:
 		case PW_TYPE_DATE:
@@ -1226,9 +1322,15 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 
 			if (attr->type != PW_TYPE_IPADDR) {
 				pair->lvalue = ntohl(lvalue);
-				ip_ntoa(pair->strvalue, pair->lvalue);
 			} else {
+				 /*
+				  *  It's an IP address, keep it in network
+				  *  byte order, and put the ASCII IP
+				  *  address or host name into the string
+				  *  value.
+				  */
 				pair->lvalue = lvalue;
+				ip_ntoa(pair->strvalue, pair->lvalue);
 			}
 
 			/*
@@ -1239,6 +1341,17 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 			        pair->flags.tag = (pair->lvalue >> 24) & 0xff;
 				pair->lvalue &= 0x00ffffff;
 			}
+
+			if (attr->type == PW_TYPE_INTEGER) {
+				DICT_VALUE *dval;
+				dval = dict_valbyattr(pair->attribute,
+						      pair->lvalue);
+				if (dval) {
+					strNcpy(pair->strvalue,
+						dval->name,
+						sizeof(pair->strvalue));
+				}
+			}
 			break;
 			
 		default:
@@ -1251,11 +1364,8 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 		
 		if (pair) {
 			debug_pair(pair);
-			if (first_pair == NULL)
-				first_pair = pair;
-			else
-				  	prev->next = pair;
-			prev = pair;
+			*tail = pair;
+			tail = &pair->next;
 		}
 
 		ptr += attrlen;
@@ -1263,8 +1373,6 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 		if (vendorlen > 0) vendorlen -= (attrlen + 2);
 	}
 
-	packet->vps = first_pair;
-
 	/*
 	 *	Merge information from the outside world into our
 	 *	random vector pool.
