@@ -1,11 +1,11 @@
 /*
  * valuepair.c	Functions to handle VALUE_PAIRs
  *
- * Version:	$Id: valuepair.c,v 1.53 2002/07/16 17:32:18 aland Exp $
+ * Version:	$Id: valuepair.c,v 1.57 2002/08/28 19:24:00 aland Exp $
  *
  */
 
-static const char rcsid[] = "$Id: valuepair.c,v 1.53 2002/07/16 17:32:18 aland Exp $";
+static const char rcsid[] = "$Id: valuepair.c,v 1.57 2002/08/28 19:24:00 aland Exp $";
 
 #include	"autoconf.h"
 
@@ -323,17 +323,15 @@ void pairmove(VALUE_PAIR **to, VALUE_PAIR **from)
 					}
 				}
 				break;
-				
 
-			   /*
-			    *  Add the new element to the list, even
-			    *  if similar ones already exist.
-			    */
+			  /*
+			   *  Add the new element to the list, even
+			   *  if similar ones already exist.
+			   */
 			default:
-			case T_OP_ADD:		/* += */
+			case T_OP_ADD: /* += */
 				break;
 			}
-		
 		}
 		if (tailfrom)
 			tailfrom->next = next;
@@ -451,41 +449,83 @@ static int gettime(const char *valstr, time_t *lvalue)
 	struct tm	*tm, s_tm;
 	char		buf[64];
 	char		*p;
-	char		*year, *month, *day;
+	char		*f[3];
 
 	time(&t);
 	tm = localtime_r(&t, &s_tm);
 
 	strNcpy(buf, valstr, sizeof(buf));
-	for (p = buf; *p; p++)
-		if (isupper((int) *p)) *p = tolower((int) *p);
 
 	p = buf;
-	day = mystrtok(&p, " \t");
-	month = mystrtok(&p, " \t");
-	year = mystrtok(&p, " \t");
-	if (!year || !month || !day) return -1;
+	f[0] = mystrtok(&p, " \t");
+	f[1] = mystrtok(&p, " \t");
+	f[2] = mystrtok(&p, " \t");
+	if (!f[0] || !f[1] || !f[2]) return -1;
 
+	/*
+	 *  The month is text, which allows us to find it easily.
+	 */
 	tm->tm_mon = 12;
-	for (i = 0; i < 12; i++) {
-		if (strncasecmp(months[i], month, 3) == 0) {
-			tm->tm_mon = i;
-			break;
+	for (i = 0; i < 3; i++) {
+		if (isalpha( (int) *f[i])) {	
+			/*
+			 *  Bubble the month to the front of the list
+			 */
+			p = f[0];
+			f[0] = f[i];
+			f[i] = p;
+
+			for (i = 0; i < 12; i++) {
+				if (strncasecmp(months[i], f[0], 3) == 0) {
+					tm->tm_mon = i;
+					break;
+				}
+			}
 		}
 	}
 
 	/* month not found? */
 	if (tm->tm_mon == 12) return -1;
 
-	tm->tm_mday = atoi(day);
+	/*
+	 *  The year may be in f[1], or in f[2]
+	 */
+	tm->tm_year = atoi(f[1]);
+	tm->tm_mday = atoi(f[2]);
+
+	if (tm->tm_year >= 1900) {
+		tm->tm_year -= 1900;
+
+	} else {
+		/*
+		 *  We can't use 2-digit years any more, they make it
+		 *  impossible to tell what's the day, and what's the year.
+		 */
+		if (tm->tm_mday < 1900) return -1;
+
+		/*
+		 *  Swap the year and the day.
+		 */
+		i = tm->tm_year;
+		tm->tm_year = tm->tm_mday - 1900;
+		tm->tm_mday = i;
+	}
+
+	/*
+	 *  If the day is out of range, die.
+	 */
 	if ((tm->tm_mday < 1) || (tm->tm_mday > 31)) {
 		return -1;
 	}
 
-	tm->tm_year = atoi(year);
-	if (tm->tm_year >= 1900) tm->tm_year -= 1900;
+	/*
+	 *  Returns -1 on error.
+	 */
+	t = mktime(tm);
+	if (t == (time_t) -1) return -1;
+
+	*lvalue = t;
 
-	*lvalue = mktime(tm);
 	return 0;
 }
 
@@ -583,16 +623,19 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 #endif
 			/* raw octets: 0x01020304... */
 		case PW_TYPE_OCTETS:
-			vp->length = 0;
 			if (strncasecmp(value, "0x", 2) == 0) {
 				u_char *us;
 				cp = value + 2;
 				us = vp->strvalue;
+				vp->length = 0;
+
 				while (*cp && vp->length < MAX_STRING_LEN) {
 					unsigned int tmp;
 					
-					if (sscanf(cp, "%02x", &tmp) != 1)
-						break;
+					if (sscanf(cp, "%02x", &tmp) != 1) {
+						librad_log("Non-hex characters at %c%c", cp[0], cp[1]);
+						return NULL;
+					}
 
 					cp += 2;
 					*(us++) = tmp;
@@ -703,15 +746,33 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 	  vp->flags.tag = tag;
 	}
 
-	/*
-	 *      For =* and !* operators, the value is irrelevant
-	 *      so we return now.
-	 */
-	if ((vp->operator == T_OP_CMP_TRUE) ||
-	    (vp->operator == T_OP_CMP_FALSE)) {
+	switch (vp->operator) {
+	default:
+		break;
+
+		/*
+		 *      For =* and !* operators, the value is irrelevant
+		 *      so we return now.
+		 */
+	case T_OP_CMP_TRUE:
+	case T_OP_CMP_FALSE:
 		vp->strvalue[0] = '\0';
 		vp->length = 0;
 	        return vp;
+		break;
+
+#ifdef HAVE_REGEX_H
+		/*
+		 *	Regular expression comparison of integer attributes
+		 *	does a STRING comparison of the names of their
+		 *	integer attributes.
+		 */
+	case T_OP_REG_EQ:	/* =~ */
+	case T_OP_REG_NE:	/* !~ */
+		if (vp->type == PW_TYPE_INTEGER) {
+			return vp;
+		}
+#endif
 	}
 
 	if (value && (pairparsevalue(vp, value) == NULL)) {
@@ -743,6 +804,7 @@ VALUE_PAIR *pairread(char **ptr, LRAD_TOKEN *eol)
 	/*  If it's a comment, then exit, as we haven't read a pair */
 	if (token == T_HASH) {
 		*eol = token;
+		librad_log("Read a comment instead of a token");
 		return NULL;
 	}
 
@@ -795,9 +857,9 @@ VALUE_PAIR *pairread(char **ptr, LRAD_TOKEN *eol)
 		 */
 	case T_BACK_QUOTED_STRING:
 		vp = pairmake(attr, NULL, token);
+		if (!vp) return vp;
 		
 		vp->flags.do_xlat = 1;
-		
 		strNcpy(vp->strvalue, value, sizeof(vp->strvalue));
 		vp->length = 0;
 		break;
