@@ -1,7 +1,7 @@
 /*
  * files.c	Read config files into memory.
  *
- * Version:     $Id: client.c,v 1.16 2002/06/17 15:34:32 aland Exp $
+ * Version:     $Id: client.c,v 1.20 2002/10/28 19:13:32 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: client.c,v 1.16 2002/06/17 15:34:32 aland Exp $";
+static const char rcsid[] = "$Id: client.c,v 1.20 2002/10/28 19:13:32 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -42,12 +42,10 @@ static const char rcsid[] = "$Id: client.c,v 1.16 2002/06/17 15:34:32 aland Exp
 #include "radiusd.h"
 #include "conffile.h"
 
-RADCLIENT *clients;
-
 /*
  *	Free a RADCLIENT list.
  */
-static void clients_free(RADCLIENT *cl)
+void clients_free(RADCLIENT *cl)
 {
 	RADCLIENT *next;
 
@@ -74,8 +72,8 @@ int read_clients_file(const char *file)
 	int lineno = 0;
 	char *p;
 
-	clients_free(clients);
-	clients = NULL;
+	clients_free(mainconfig.clients);
+	mainconfig.clients = NULL;
 
 	if ((fp = fopen(file, "r")) == NULL) {
 		/* The clients file is no longer required.  All configuration
@@ -166,6 +164,7 @@ int read_clients_file(const char *file)
 		 *	It should be OK now, let's create the buffer.
 		 */
 		c = rad_malloc(sizeof(RADCLIENT));
+		memset(c, 0, sizeof(*c));
 
 		c->ipaddr = ip_getaddr(hostnm);
 		if (c->ipaddr == INADDR_NONE) {
@@ -174,7 +173,7 @@ int read_clients_file(const char *file)
 			return -1;
 		}
 		c->netmask = htonl(mask);
-		c->ipaddr &= mask;
+		c->ipaddr &= c->netmask; /* addr & mask are in network order */
 
 		strcpy((char *)c->secret, secret);
 		strcpy(c->shortname, shortnm);
@@ -183,15 +182,36 @@ int read_clients_file(const char *file)
 		 *	Only do DNS lookups for machines.  Just print
 		 *	the network as the long name.
 		 */
-		if (c->netmask == ~0U) {
+		if ((~mask) == 0) {
+			NAS *nas;
 			ip_hostname(c->longname, sizeof(c->longname), c->ipaddr);
+
+			/*
+			 *	Pull information over from the NAS.
+			 */
+			nas = nas_find(c->ipaddr);
+			if (nas) {
+				/*
+				 *	No short name in the 'clients' file,
+				 *	try copying one over from the
+				 *	'naslist' file.
+				 */
+				if (c->shortname[0] == '\0') {
+					strcpy(c->shortname, nas->shortname);
+				}
+
+				/*
+				 *  Copy the nastype over, too.
+				 */
+				strcpy(c->nastype, nas->nastype);
+			}
 		} else {
 			hostnm[strlen(hostnm)] = '/';
 			strNcpy(c->longname, hostnm, sizeof(c->longname));
 		}
 
-		c->next = clients;
-		clients = c;
+		c->next = mainconfig.clients;
+		mainconfig.clients = c;
 	}
 	fclose(fp);
 
@@ -207,7 +227,7 @@ RADCLIENT *client_find(uint32_t ipaddr)
 	RADCLIENT *cl;
 	RADCLIENT *match = NULL;
 
-	for (cl = clients; cl; cl = cl->next) {
+	for (cl = mainconfig.clients; cl; cl = cl->next) {
 		if ((ipaddr & cl->netmask) == cl->ipaddr) {
 			if ((!match) ||
 			    (ntohl(cl->netmask) > ntohl(match->netmask))) {
@@ -228,7 +248,7 @@ void client_walk(void)
 	RADCLIENT *cl;
 	char host_ipaddr[16];
 
-	for (cl = clients; cl != NULL; cl = cl->next)
+	for (cl = mainconfig.clients; cl != NULL; cl = cl->next)
 		radlog(L_ERR, "client: client_walk: %s\n",
 				ip_ntoa(host_ipaddr, cl->ipaddr));
 }
