@@ -1,7 +1,7 @@
 /*
  * log.c	Logging module.
  *
- * Version:	$Id: log.c,v 1.32 2002/07/22 23:10:06 ramoore Exp $
+ * Version:	$Id: log.c,v 1.34 2002/10/29 22:55:27 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2001  Chad Miller <cmiller@surfsouth.com>
  */
 
-static const char rcsid[] = "$Id: log.c,v 1.32 2002/07/22 23:10:06 ramoore Exp $";
+static const char rcsid[] = "$Id: log.c,v 1.34 2002/10/29 22:55:27 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -32,9 +32,7 @@ static const char rcsid[] = "$Id: log.c,v 1.32 2002/07/22 23:10:06 ramoore Exp $
 #include <string.h>
 #include <stdarg.h>
 #include <unistd.h>
-#include <pwd.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <errno.h>
 
 #include "radiusd.h"
@@ -43,71 +41,6 @@ static const char rcsid[] = "$Id: log.c,v 1.32 2002/07/22 23:10:06 ramoore Exp $
 #	include <syslog.h>
 #endif
 
-static int r_mkdir(const char *);
-
-
-static int r_mkdir(const char *part) {
-	char *ptr, parentdir[500];
-	struct stat st;
-
-	if (stat(part, &st) == 0)
-		return(0);
-
-	ptr = strrchr(part, '/');
-
-	if (ptr == part)
-		return(0);
-
-	snprintf(parentdir, (ptr - part)+1, "%s", part);
-
-	if (r_mkdir(parentdir) != 0)
-		return(1);
-
-	if (mkdir(part, 0770) != 0) {
-		fprintf(stderr, "mkdir(%s) error: %s\n", part, strerror(errno));
-		return(1);
-	}
-
-	fprintf(stderr, "Created directory %s\n", part);
-
-	return(0);
-}
-		
-
-int radlogdir_iswritable(const char *effectiveuser) {
-	struct passwd *pwent;
-
-	if (radlog_dir[0] != '/')
-		return(0);
-
-	if (r_mkdir(radlog_dir) != 0)
-		return(1);
-
-	/* FIXME: do we have this function? */
-	if (strstr(radlog_dir, "radius") == NULL)
-		return(0);
-
-	/* we have a logdir that mentions 'radius', so it's probably 
-	 * safe to chown the immediate directory to be owned by the normal 
-	 * process owner. we gotta do it before we give up root.  -chad
-	 */
-	
-	if (!effectiveuser) {
-		return 1;
-	}
-
-	pwent = getpwnam(effectiveuser);
-
-	if (pwent == NULL) /* uh oh! */
-		return(1);
-
-	if (chown(radlog_dir, pwent->pw_uid, -1) != 0)
-		return(1);
-
-	return(0);
-}
-
-
 /*
  *	Log the message to the logfile. Include the severity and
  *	a time stamp.
@@ -115,7 +48,6 @@ int radlogdir_iswritable(const char *effectiveuser) {
 int vradlog(int lvl, const char *fmt, va_list ap)
 {
 	FILE *msgfd = NULL;
-	const char *s = ": ";
 	unsigned char *p;
 	char buffer[8192];
 	int len;
@@ -139,8 +71,7 @@ int vradlog(int lvl, const char *fmt, va_list ap)
 
 	} else if (radlog_dest != RADLOG_SYSLOG) {
 
-	        sprintf(buffer, "%.1000s/%.1000s", radlog_dir, RADIUS_LOG);
-		if ((msgfd = fopen(buffer, "a")) == NULL) {
+		if ((msgfd = fopen(mainconfig.log_file, "a")) == NULL) {
 		         fprintf(stderr, "%s: Couldn't open %s for logging: %s\n",
 				 progname, buffer, strerror(errno));
 				
@@ -159,7 +90,9 @@ int vradlog(int lvl, const char *fmt, va_list ap)
 	} else
 #endif
 	{
+		const char *s = ": ";
 		time_t timeval;
+
 		timeval = time(NULL);
 		ctime_r(&timeval, buffer);
 
