@@ -1,7 +1,7 @@
 /*
  * modules.c	Radius module support.
  *
- * Version:	$Id: modules.c,v 1.66 2002/06/14 15:17:38 aland Exp $
+ * Version:	$Id: modules.c,v 1.70 2002/10/21 15:41:07 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
 
-static const char rcsid[] = "$Id: modules.c,v 1.66 2002/06/14 15:17:38 aland Exp $";
+static const char rcsid[] = "$Id: modules.c,v 1.70 2002/10/21 15:41:07 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -129,7 +129,10 @@ static void instance_list_free(module_instance_t **i)
 	*i = NULL;
 }
 
-static void module_list_free(void)
+/*
+ *	Remove all of the modules.
+ */
+int detach_modules(void)
 {
 	module_list_t *ml, *next;
 	int i;
@@ -154,6 +157,8 @@ static void module_list_free(void)
 	}
 
 	module_list = NULL;
+
+	return 0;
 }
 
 /*
@@ -202,7 +207,7 @@ static int new_sectiontype_value(const char *name,int type)
 static module_list_t *linkto_module(const char *module_name,
 		const char *cffilename, int cflineno)
 {
-	module_list_t **last, *node;
+	module_list_t *node;
 	lt_dlhandle handle;
 	char module_struct[256];
 	char *p;
@@ -211,7 +216,6 @@ static module_list_t *linkto_module(const char *module_name,
 	 *	Look through the global module library list for the
 	 *	named module.
 	 */
-	last = &module_list;
 	for (node = module_list; node != NULL; node = node->next) {
 		/*
 		 *	Found the named module.  Return it.
@@ -219,10 +223,6 @@ static module_list_t *linkto_module(const char *module_name,
 		if (strcmp(node->name, module_name) == 0)
 			return node;
 
-		/*
-		 *	Keep a pointer to the last entry to update...
-		 */
-		last = &node->next;
 	}
 
 	/*
@@ -273,7 +273,8 @@ static module_list_t *linkto_module(const char *module_name,
 
 	DEBUG("Module: Loaded %s ", node->module->name);
 
-	*last = node;
+	node->next = module_list;
+	module_list = node;
 
 	return node;
 }
@@ -333,7 +334,7 @@ module_instance_t *find_module_instance(const char *instname)
 		name1 = cf_section_name1(inst_cs);
 		name2 = cf_section_name2(inst_cs);
 		if ( (name2 && !strcmp(name2, instname)) ||
-				(!name2 && !strcmp(name1, instname)) )
+		     (!name2 && !strcmp(name1, instname)) )
 			break;
 	}
 	if (inst_cs == NULL) {
@@ -364,7 +365,9 @@ module_instance_t *find_module_instance(const char *instname)
 
 	}
 
-	/* XXX "radiusd.conf" is wrong here; must find cf filename */
+	/*
+	 *  FIXME: "radiusd.conf" is wrong here; must find cf filename
+	 */
 	node->entry = linkto_module(module_name, "radiusd.conf",
 				    cf_section_lineno(inst_cs));
 	if (!node->entry) {
@@ -377,7 +380,7 @@ module_instance_t *find_module_instance(const char *instname)
 	 *	Call the module's instantiation routine.
 	 */
 	if ((node->entry->module->instantiate) &&
-			((node->entry->module->instantiate)(inst_cs,
+	    ((node->entry->module->instantiate)(inst_cs,
 			&node->insthandle) < 0)) {
 		radlog(L_ERR|L_CONS,
 				"radiusd.conf[%d]: %s: Module instantiation failed.\n",
@@ -542,7 +545,7 @@ static void load_component_section(CONF_SECTION *cs, int comp, const char *filen
 			scs = cf_itemtosection(modref);
 
 			if (strcmp(cf_section_name1(scs),
-					subcomponent_names[comp]) == 0) {
+				   subcomponent_names[comp]) == 0) {
 				load_subcomponent_section(scs, comp, filename);
 				continue;
 			}
@@ -595,6 +598,10 @@ int setup_modules(void)
 {
 	int comp;
 	CONF_SECTION *cs;
+
+	/*
+	 *  FIXME: This should be pulled from somewhere else.
+	 */
 	const char *filename="radiusd.conf";
 
 	/*
@@ -635,9 +642,43 @@ int setup_modules(void)
 		}
 
 	} else {
-		module_list_free();
+		detach_modules();
 	}
 
+	/*
+	 *  Look for the 'instantiate' section, which tells us
+	 *  the instantiation order of the modules, and also allows
+	 *  us to load modules with no authorize/authenticate/etc.
+	 *  sections.
+	 */
+	cs = cf_section_find("instantiate");
+	if (cs != NULL) {
+		CONF_ITEM *ci;
+		CONF_PAIR *cp;
+		module_instance_t *module;
+		const char *name;
+
+		/*
+		 *  Loop over the items in the 'instantiate' section.
+		 */
+		for (ci=cf_item_find_next(cs, NULL); ci != NULL; ci=cf_item_find_next(cs, ci)) {
+			if (cf_item_is_section(ci)) {
+				radlog(L_ERR|L_CONS,
+				       "%s[%d] Subsection for module instantiate is not allowed\n", filename,
+				       
+				       cf_section_lineno(cf_itemtosection(ci)));
+				exit(1);
+			}
+	
+			cp = cf_itemtopair(ci);
+			name = cf_pair_attr(cp);
+			module = find_module_instance(name);
+			if (!module) {
+				exit(1);
+			}
+		} /* loop over items in the subsection */
+	} /* if there's an 'instantiate' section. */
+
 	/*
 	 *	Loop over all of the known components, finding their
 	 *	configuration section, and loading it.
