@@ -26,7 +26,7 @@
  *
  */
 char radrelay_rcsid[] =
-"$Id: radrelay.c,v 1.2 2002/07/08 18:23:42 aland Exp $";
+"$Id: radrelay.c,v 1.8 2002/10/30 20:38:18 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -73,6 +73,7 @@ uint32_t myip = INADDR_ANY;
 int proxy_retry_delay = RETRY_DELAY;
 int proxy_retry_count = RETRY_COUNT;
 int proxy_dead_time;
+int max_proxies = MAX_PROXIES;
 int log_stripped_names;
 struct main_config_t mainconfig;
 
@@ -256,17 +257,13 @@ int read_one(FILE *fp, struct relay_request *r_req)
 			}
 			if (!skip) {
 				vp = NULL;
-				/*
-				 * FIXME: We should check that the attribute
-				 * we've read atleast remotely ressembles
-				 * a correct attribute, otherwise we run the
-				 * risk of segfaulting in userparse().
-				 */
-				if (userparse(buf, &vp) >= 0 &&
+				if (userparse(buf, &vp) > 0 &&
 				    (vp->attribute < 256 ||
 				     vp->attribute > 65535) &&
 				    vp->attribute != PW_VENDOR_SPECIFIC) {
 					pairadd(&(r_req->req->vps), vp);
+				} else {
+				  pairfree(&vp);
 				}
 			}
 		}
@@ -620,9 +617,14 @@ void loop(struct relay_misc *r_args)
  */
 int find_shortname(char *shortname, char **host, char **secret)
 {
-	CONF_SECTION *cs;
+	CONF_SECTION *maincs, *cs;
 
-	if (read_radius_conf_file() < 0) {
+	/*
+	 *	Ensure that the configuration is initialized.
+	 */
+	memset(&mainconfig, 0, sizeof(mainconfig));
+
+	if ((maincs = read_radius_conf_file()) == NULL) {
 		fprintf(stderr, "Error reading radiusd.conf\n");
 		exit(1);
 	}
@@ -630,7 +632,7 @@ int find_shortname(char *shortname, char **host, char **secret)
 	/*
 	 * Find the first 'client' section.
 	 */
-	cs = cf_section_find("client");
+	cs = cf_section_sub_find(maincs, "client");
 	if (cs) {
 		c_shortname = cf_section_value_find(cs, "shortname");
 		c_secret = cf_section_value_find(cs, "secret");
@@ -792,6 +794,14 @@ int main(int argc, char **argv)
 			usage();
 			break;
 	}
+
+	/*
+	 *	No detail file: die.
+	 */
+	if (argc == optind) {
+		usage();
+	}
+
 	argc -= (optind - 1);
 	argv += (optind - 1);
 	if (shortname && server_name)
