@@ -1,7 +1,7 @@
 /*
  * session.c	session management
  *
- * Version:	$Id: session.c,v 1.12 2002/05/13 19:08:06 aland Exp $
+ * Version:	$Id: session.c,v 1.15 2002/10/18 18:44:30 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -166,76 +166,81 @@ int session_zap(int sockfd, uint32_t nasaddr, int port, const char *user,
 }
 
 
-/*
- *	Timeout handler (10 secs)
- */
-static volatile int got_alrm;
-static void alrm_handler(int s)
-{
-	(void)s;
-	got_alrm = 1;
-}
-
 /*
  *	Check one terminal server to see if a user is logged in.
  */
 int rad_check_ts(uint32_t nasaddr, int portnum, const char *user,
 		 const char *session_id)
 {
-	int	pid, st, e;
+	pid_t	pid, child_pid;
+	int	status;
 	int	n;
-	NAS	*nas;
 	char	address[16];
 	char	port[8];
-	void	(*handler)(int);
+	RADCLIENT *cl;
 
 	/*
 	 *	Find NAS type.
 	 */
-	if ((nas = nas_find(nasaddr)) == NULL) {
-	/*	radlog(L_ERR, "Accounting: unknown NAS, so trusting radutmp");	*/
+	cl = client_find(nasaddr);
+	if (!cl) {
+		/*
+		 *  Unknown NAS, so trusting radutmp.
+		 */
 		return 1;
 	}
-	if (nas->nastype == "other") {
-	/*	radlog(L_ERR, "Accounting: NAS type is 'other', so trusting radutmp");	*/
+
+	/*
+	 *  No nastype, or nas type 'other', trust radutmp.
+	 */
+	if ((cl->nastype[0] == '\0') ||
+	    (strcmp(cl->nastype, "other") == 0)) {
 		return 1;
 	}
 
 	/*
 	 *	Fork.
 	 */
-	handler = signal(SIGCHLD, SIG_DFL);
-	if ((pid = fork()) < 0) {
+	if ((pid = rad_fork(1)) < 0) { /* do wait for the fork'd result */
 		radlog(L_ERR, "Accounting: fork: %s", strerror(errno));
-		signal(SIGCHLD, handler);
 		return -1;
 	}
 
 	if (pid > 0) {
+		int found = 0;
+
 		/*
 		 *	Parent - Wait for checkrad to terminate.
 		 *	We timeout in 10 seconds.
 		 */
-		got_alrm = 0;
-		signal(SIGALRM, alrm_handler);
-		alarm(10);
-		while((e = waitpid(pid, &st, 0)) != pid)
-			if (e < 0 && (errno != EINTR || got_alrm))
+		child_pid = -1;
+		for (n = 0; n < 10; n++) {
+			sleep(1);
+			child_pid = rad_waitpid(pid, &status, WNOHANG);
+			if ((child_pid <= 0) || (child_pid == pid)) {
+				found = 1;
 				break;
-		alarm(0);
-		signal(SIGCHLD, handler);
-		if (got_alrm) {
+			}
+		}
+
+		/*
+		 *  It's taking too long.  Kill it.
+		 */
+		if (!found) {
 			kill(pid, SIGTERM);
 			sleep(1);
 			kill(pid, SIGKILL);
 			radlog(L_ERR, "Check-TS: timeout waiting for checkrad");
+			rad_waitpid(pid, &status, WNOHANG); /* to be safe */
 			return 2;
 		}
-		if (e < 0) {
+
+		if (child_pid < 0) {
 			radlog(L_ERR, "Check-TS: unknown error in waitpid()");
 			return 2;
 		}
-		return WEXITSTATUS(st);
+
+		return WEXITSTATUS(status);
 	}
 
 	/*
@@ -251,13 +256,13 @@ int rad_check_ts(uint32_t nasaddr, int portnum, const char *user,
 	/* OS/2 can't directly execute scripts then we call the command
 	   processor to execute checkrad
 	*/
-	execl(getenv("COMSPEC"), "", "/C","checkrad",nas->nastype, address, port,
+	execl(getenv("COMSPEC"), "", "/C","checkrad", cl->nastype, address, port,
 		user, session_id, NULL);
 #else
-	execl(CHECKRAD, "checkrad",nas->nastype, address, port,
+	execl(mainconfig.checkrad, "checkrad", cl->nastype, address, port,
 		user, session_id, NULL);
 #endif
-	radlog(L_ERR, "Check-TS: exec %s: %s", CHECKRAD, strerror(errno));
+	radlog(L_ERR, "Check-TS: exec %s: %s", mainconfig.checkrad, strerror(errno));
 
 	/*
 	 *	Exit - 2 means "some error occured".
