@@ -1,7 +1,7 @@
 /*
  * threads.c	request threading support
  *
- * Version:	$Id: threads.c,v 1.45 2002/07/08 18:34:58 aland Exp $
+ * Version:	$Id: threads.c,v 1.49 2002/10/28 17:23:28 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@
 #include "conffile.h"
 
 static const char rcsid[] =
-"$Id: threads.c,v 1.45 2002/07/08 18:34:58 aland Exp $";
+"$Id: threads.c,v 1.49 2002/10/28 17:23:28 aland Exp $";
 
 #define SEMAPHORE_LOCKED	(0)
 #define SEMAPHORE_UNLOCKED	(1)
@@ -72,7 +72,7 @@ typedef struct THREAD_HANDLE {
 	int thread_num;
 	sem_t semaphore;
 	int status;
-	int request_count;
+	unsigned int request_count;
 	time_t timestamp;
 	REQUEST *request;
 	RAD_REQUEST_FUNP fun;
@@ -95,6 +95,7 @@ typedef struct THREAD_POOL {
 	int min_spare_threads;
 	int max_spare_threads;
 	int max_requests_per_thread;
+	unsigned long request_count;
 	time_t time_last_spawned;
 	int cleanup_delay;
 } THREAD_POOL;
@@ -165,6 +166,7 @@ static void *request_handler_thread(void *arg)
 	sigaddset(&set, SIGHUP);
 	sigaddset(&set, SIGINT);
 	sigaddset(&set, SIGQUIT);
+	sigaddset(&set, SIGTERM);
 	pthread_sigmask(SIG_BLOCK, &set, NULL);
 #endif
 	
@@ -580,6 +582,7 @@ int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 	found->request = request;
 	found->fun = fun;
 	found->request_count++;
+	thread_pool.request_count++;
 	sem_post(&found->semaphore);
 	thread_pool.active_threads = active_threads;
 
@@ -721,6 +724,8 @@ int thread_pool_clean(time_t now)
 	return 0;
 }
 
+static int exec_initialized = FALSE;
+
 /*
  *	Initialize the stuff for keeping track of child processes.
  */
@@ -742,6 +747,8 @@ void rad_exec_init(void)
 		forkers[i].child_pid = -1;
 		forkers[i].status = 0;
 	}
+
+	exec_initialized = TRUE;
 }
 
 /*
@@ -761,7 +768,7 @@ pid_t rad_fork(int exec_wait)
 	 *	Or, there no NO threads, so we can just do the fork
 	 *	thing.
 	 */
-	if (!exec_wait || !pool_initialized) {
+	if (!exec_wait || !exec_initialized) {
 		return fork();
 	}
 
@@ -935,6 +942,12 @@ int rad_savepid(pid_t pid, int status)
 			 */
 			forkers[i].status = status;
 			sem_post(&forkers[i].child_done);
+
+			/*
+			 *  If the child is more than 60 seconds out
+			 *  of date, then delete it.
+			 */
+
 			return 0;		
 		}
 	}
