@@ -1,7 +1,7 @@
 /*
  * rlm_chap.c
  *
- * Version:  $Id: rlm_chap.c,v 1.3 2002/05/29 23:46:51 fcusack Exp $
+ * Version:  $Id: rlm_chap.c,v 1.4 2002/08/24 16:54:56 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
 #include "modules.h"
 #include "conffile.h"
 
-static const char rcsid[] = "$Id: rlm_chap.c,v 1.3 2002/05/29 23:46:51 fcusack Exp $";
+static const char rcsid[] = "$Id: rlm_chap.c,v 1.4 2002/08/24 16:54:56 aland Exp $";
 
 static int chap_authorize(void *instance, REQUEST *request)
 {
@@ -143,7 +143,10 @@ module_t rlm_chap = {
 		chap_authorize,	 	/* authorization */
 		NULL,			/* preaccounting */
 		NULL,			/* accounting */
-		NULL			/* checksimul */
+		NULL,			/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		NULL			/* post-auth */
 	},
 	NULL,				/* detach */
 	NULL,				/* destroy */
