@@ -1,7 +1,7 @@
 /*
  * eap.c    rfc2284 & rfc2869 implementation
  *
- * Version:     $Id: eap.c,v 1.7 2002/04/26 21:35:50 raghu Exp $
+ * Version:     $Id: eap.c,v 1.10 2002/09/18 21:07:42 cparker Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -414,16 +414,33 @@ int eap_compose(REQUEST *request, EAP_PACKET *reply)
 {
 	uint16_t eap_len, len;
 	VALUE_PAIR *eap_msg;
+	EAP_PACKET *rq;
 	VALUE_PAIR *vp;
 	eap_packet_t *eap_packet;
 	unsigned char 	*ptr;
 
 	/*
-	 * Id is expected to be set by the EAP-Type, if not,
-	 * currently RADIUS request Id is used.
-	 * Ideally, a unique Id should be generated.
+	 * ID serves to suppport request/response
+	 * retransmission in the EAP layer and as
+	 * such must be different for 'adjacent'
+	 * packets except in case of success/failure-replies.
+	 *
+	 * RFC2716 (EAP_TLS) requires this to be
+	 * incremented, RFC2284 only makes the above-
+	 * mentioned restriction.
 	 */
-	if (reply->id == 0) reply->id = request->packet->id;
+	eap_msg = pairfind(request->packet->vps, PW_EAP_MESSAGE);
+	rq = (EAP_PACKET *)eap_msg->strvalue;
+	reply->id = rq->id;
+
+	switch (reply->code) {
+		case PW_EAP_SUCCESS:
+		case PW_EAP_FAILURE:
+	    		break;
+
+		default:
+	    		++reply->id;
+	}
 
 	if (eap_wireformat(reply) == EAP_INVALID) {
 		return EAP_INVALID;
@@ -751,6 +768,7 @@ EAP_HANDLER *eap_handler(EAP_HANDLER **list, eap_packet_t **eap_packet_p, REQUES
 	}
 
 	handler->timestamp = time(NULL);
+	handler->reply_vps = &(request->reply->vps);
 	return handler;
 }
 
