@@ -1,7 +1,7 @@
 /*
  * rlm_eap.c  contains handles that are called from modules.
  *
- * Version:     $Id: rlm_eap.c,v 1.7 2002/06/10 11:08:28 fcusack Exp $
+ * Version:     $Id: rlm_eap.c,v 1.8 2002/08/24 16:54:57 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -344,7 +344,10 @@ module_t rlm_eap = {
 		eap_authorize,		/* authorization */
 		NULL,			/* preaccounting */
 		NULL,			/* accounting */
-		NULL			/* checksimul */
+		NULL,			/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		NULL			/* post-auth */
 	},
 	eap_detach,			/* detach */
 	NULL,				/* destroy */
