@@ -1,7 +1,7 @@
 /*
  * rlm_ippool.c
  *
- * Version:  $Id: rlm_ippool.c,v 1.7 2002/07/16 16:30:59 aland Exp $
+ * Version:  $Id: rlm_ippool.c,v 1.12 2002/10/11 13:26:20 kkalev Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -28,6 +28,12 @@
  * - Check the return value of a gdbm_fetch() we didn't check
  * - Change the nas entry in the ippool_key structure from uint32 to string[64]
  *   That should allow us to also use the NAS-Identifier attribute
+ * Sep 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
+ * - Move from authorize to post-auth
+ * - Use mutex locks when accessing the gdbm files
+ * - Fail if we don't find nas port information
+ * Oct 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
+ * - Do a memset(0) on the key.nas before doing searches. Nusty bug
  */
 
 #include "config.h"
@@ -59,14 +65,10 @@
 #define GDBM_IPPOOL_OPTS (GDBM_SYNCOPT)
 #endif
 
-#ifndef HAVE_GDBM_FDESC
-#define gdbm_fdesc(foo) (-1)
-#endif
-
 #define ALL_ONES 4294967295
 #define MAX_NAS_NAME_SIZE 64
 
-static const char rcsid[] = "$Id: rlm_ippool.c,v 1.7 2002/07/16 16:30:59 aland Exp $";
+static const char rcsid[] = "$Id: rlm_ippool.c,v 1.12 2002/10/11 13:26:20 kkalev Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -85,8 +87,8 @@ typedef struct rlm_ippool_t {
 	int cache_size;
 	GDBM_FILE gdbm;
 	GDBM_FILE ip;
-	int fd;
-	int ip_fd;
+	pthread_mutex_t session_mutex;
+	pthread_mutex_t ip_mutex;
 } rlm_ippool_t;
 
 typedef struct ippool_info {
@@ -176,6 +178,7 @@ static int ippool_instantiate(CONF_SECTION *conf, void **instance)
 		free(data);
 		return -1;
 	}
+	
 	data->gdbm = gdbm_open(data->session_db, sizeof(int),
 			GDBM_WRCREAT | GDBM_IPPOOL_OPTS, 0600, NULL);
 	if (data->gdbm == NULL) {
@@ -183,8 +186,6 @@ static int ippool_instantiate(CONF_SECTION *conf, void **instance)
 				data->session_db, strerror(errno));
 		return -1;
 	}
-	if (data->fd >= 0) data->fd = gdbm_fdesc(data->gdbm);
-
 	data->ip = gdbm_open(data->ip_index, sizeof(int),
 			GDBM_WRCREAT | GDBM_IPPOOL_OPTS, 0600, NULL);
 	if (data->ip == NULL) {
@@ -192,16 +193,12 @@ static int ippool_instantiate(CONF_SECTION *conf, void **instance)
 				data->ip_index, strerror(errno));
 		return -1;
 	}
-	if (data->ip_fd >= 0) data->ip_fd = gdbm_fdesc(data->ip);
-
 	if (gdbm_setopt(data->gdbm, GDBM_CACHESIZE, &cache_size, sizeof(int)) == -1)
 		radlog(L_ERR, "rlm_ippool: Failed to set cache size");
 	if (gdbm_setopt(data->ip, GDBM_CACHESIZE, &cache_size, sizeof(int)) == -1)
 		radlog(L_ERR, "rlm_ippool: Failed to set cache size");
 
-	if (data->fd >= 0) rad_lockfd(data->fd, sizeof(int));
 	key_datum = gdbm_firstkey(data->gdbm);
-	if (data->fd >= 0) rad_unlockfd(data->fd, sizeof(int));
 	if (key_datum.dptr == NULL){
 			/*
 			 * If the database does not exist initialize it.
@@ -237,9 +234,7 @@ static int ippool_instantiate(CONF_SECTION *conf, void **instance)
 			data_datum.dptr = (ippool_info *) &entry;
 			data_datum.dsize = sizeof(ippool_info);
 
-			if (data->fd >= 0) rad_lockfd(data->fd, sizeof(int));
 			rcode = gdbm_store(data->gdbm, key_datum, data_datum, GDBM_REPLACE);
-			if (data->fd >= 0) rad_unlockfd(data->fd, sizeof(int));
 			if (rcode < 0) {
 				radlog(L_ERR, "rlm_ippool: Failed storing data to %s: %s",
 						data->session_db, gdbm_strerror(gdbm_errno));
@@ -258,6 +253,8 @@ static int ippool_instantiate(CONF_SECTION *conf, void **instance)
 	pool_name = cf_section_name2(conf);
 	if (pool_name != NULL)
 		data->name = strdup(pool_name);
+	pthread_mutex_init(&data->session_mutex, NULL);
+	pthread_mutex_init(&data->ip_mutex, NULL);
 
 	*instance = data;
 	
@@ -316,20 +313,22 @@ static int ippool_accounting(void *instance, REQUEST *request)
 			return RLM_MODULE_NOOP;
 	}
 
-	strncpy(key.nas,nas,MAX_NAS_NAME_SIZE - 1);
+	memset(key.nas,0,MAX_NAS_NAME_SIZE);
+	strncpy(key.nas,nas,MAX_NAS_NAME_SIZE -1 );
 	key.port = port;
+	DEBUG("rlm_ippool: Searching for an entry for nas/port: %s/%d",key.nas,key.port);
 	key_datum.dptr = (ippool_key *) &key;
 	key_datum.dsize = sizeof(ippool_key);
 
-	if (data->fd >= 0) rad_lockfd(data->fd, sizeof(int));
+	pthread_mutex_lock(&data->session_mutex);
 	data_datum = gdbm_fetch(data->gdbm, key_datum);
-	if (data->fd >= 0) rad_unlockfd(data->fd, sizeof(int));
+	pthread_mutex_unlock(&data->session_mutex);
 	if (data_datum.dptr != NULL){
 
 		/*
 		 * If the entry was found set active to zero
 		 */
-		memcpy(&entry, data_datum.dptr, sizeof(int));
+		memcpy(&entry, data_datum.dptr, sizeof(ippool_info));
 		free(data_datum.dptr);
 		DEBUG("rlm_ippool: Deallocated entry for ip/port: %s/%d",ip_ntoa(str,entry.ipaddr),port);
 		entry.active = 0;
@@ -337,9 +336,9 @@ static int ippool_accounting(void *instance, REQUEST *request)
 		data_datum.dptr = (ippool_info *) &entry;
 		data_datum.dsize = sizeof(ippool_info);
 
-		if (data->fd >= 0) rad_lockfd(data->fd, sizeof(int));
+		pthread_mutex_lock(&data->session_mutex);
 		rcode = gdbm_store(data->gdbm, key_datum, data_datum, GDBM_REPLACE);
-		if (data->fd >= 0) rad_unlockfd(data->fd, sizeof(int));
+		pthread_mutex_unlock(&data->session_mutex);
 		if (rcode < 0) {
 			radlog(L_ERR, "rlm_ippool: Failed storing data to %s: %s",
 					data->session_db, gdbm_strerror(gdbm_errno));
@@ -351,9 +350,9 @@ static int ippool_accounting(void *instance, REQUEST *request)
 		 */
 		key_datum.dptr = (uint32_t *) &entry.ipaddr;
 		key_datum.dsize = sizeof(uint32_t);
-		if (data->ip_fd >= 0) rad_lockfd(data->ip_fd, sizeof(int));
+		pthread_mutex_lock(&data->ip_mutex);
 		data_datum = gdbm_fetch(data->ip, key_datum);
-		if (data->ip_fd >= 0) rad_unlockfd(data->ip_fd, sizeof(int));
+		pthread_mutex_unlock(&data->ip_mutex);
 		if (data_datum.dptr != NULL){
 			memcpy(&num, data_datum.dptr, sizeof(int));
 			free(data_datum.dptr);
@@ -362,9 +361,9 @@ static int ippool_accounting(void *instance, REQUEST *request)
 				DEBUG("rlm_ippool: num: %d",num);
 				data_datum.dptr = (int *) &num;
 				data_datum.dsize = sizeof(int);
-				if (data->ip_fd >= 0) rad_lockfd(data->ip_fd, sizeof(int));
+				pthread_mutex_lock(&data->ip_mutex);
 				rcode = gdbm_store(data->ip, key_datum, data_datum, GDBM_REPLACE);
-				if (data->ip_fd >= 0) rad_unlockfd(data->ip_fd, sizeof(int));
+				pthread_mutex_unlock(&data->ip_mutex);
 				if (rcode < 0) {
 					radlog(L_ERR, "rlm_ippool: Failed storing data to %s: %s",
 							data->ip_index, gdbm_strerror(gdbm_errno));
@@ -373,11 +372,13 @@ static int ippool_accounting(void *instance, REQUEST *request)
 			}
 		}
 	}
+	else
+		DEBUG("rlm_ippool: Entry not found");
 
 	return RLM_MODULE_OK;
 }
 
-static int ippool_authorize(void *instance, REQUEST *request)
+static int ippool_postauth(void *instance, REQUEST *request)
 {
 	rlm_ippool_t *data = (rlm_ippool_t *) instance;
 	int port = 0;
@@ -406,11 +407,13 @@ static int ippool_authorize(void *instance, REQUEST *request)
 		if (data->name == NULL || strcmp(data->name,vp->strvalue))
 			return RLM_MODULE_NOOP;
 	} else {
+		DEBUG("rlm_ippool: Could not find Pool-Name attribute.");
 		return RLM_MODULE_NOOP;
 	}
 
 	/*
 	 * Get the nas ip address
+	 * If not fail
 	 */
 	if ((vp = pairfind(request->packet->vps, PW_NAS_IP_ADDRESS)) != NULL)
 		strncpy(nas, vp->strvalue, MAX_NAS_NAME_SIZE - 1);
@@ -431,64 +434,69 @@ static int ippool_authorize(void *instance, REQUEST *request)
 
 	/*
 	 * Find the port
+	 * If not fail
 	 */
-	if ((vp = pairfind(request->packet->vps, PW_NAS_PORT_ID)) != NULL){
+	if ((vp = pairfind(request->packet->vps, PW_NAS_PORT_ID)) != NULL)
 		port = vp->lvalue;
+	else{
+		DEBUG("rlm_ippool: Could not find port information.");
+		return RLM_MODULE_NOOP;
+	}
 
-		strncpy(key.nas,nas,MAX_NAS_NAME_SIZE -1 );
-		key.port = port;	
-		DEBUG("rlm_ippool: Searching for an entry for nas/port: %s/%d",nas,port);
-		key_datum.dptr = (ippool_key *) &key;
-		key_datum.dsize = sizeof(ippool_key);
+	memset(key.nas,0,MAX_NAS_NAME_SIZE);
+	strncpy(key.nas,nas,MAX_NAS_NAME_SIZE -1 );
+	key.port = port;	
+	DEBUG("rlm_ippool: Searching for an entry for nas/port: %s/%d",key.nas,key.port);
+	key_datum.dptr = (ippool_key *) &key;
+	key_datum.dsize = sizeof(ippool_key);
 
-		if (data->fd >= 0) rad_lockfd(data->fd, sizeof(int));
-		data_datum = gdbm_fetch(data->gdbm, key_datum);
-		if (data->fd >= 0) rad_unlockfd(data->fd, sizeof(int));
-		if (data_datum.dptr != NULL){
-			/*
-			 * If there is a corresponding entry in the database with active=1 it is stale.
-			 * Set active to zero
-			 */
-			memcpy(&entry, data_datum.dptr, sizeof(ippool_info));
-			free(data_datum.dptr);
-			if (entry.active){
-				DEBUG("rlm_ippool: Found a stale entry for ip/port: %s/%d",ip_ntoa(str,entry.ipaddr),port);
-				entry.active = 0;
+	pthread_mutex_lock(&data->session_mutex);
+	data_datum = gdbm_fetch(data->gdbm, key_datum);
+	pthread_mutex_unlock(&data->session_mutex);
+	if (data_datum.dptr != NULL){
+		/*
+		 * If there is a corresponding entry in the database with active=1 it is stale.
+		 * Set active to zero
+		 */
+		memcpy(&entry, data_datum.dptr, sizeof(ippool_info));
+		free(data_datum.dptr);
+		if (entry.active){
+			DEBUG("rlm_ippool: Found a stale entry for ip/port: %s/%d",ip_ntoa(str,entry.ipaddr),port);
+			entry.active = 0;
 
-				data_datum.dptr = (ippool_info *) &entry;
-				data_datum.dsize = sizeof(ippool_info);
+			data_datum.dptr = (ippool_info *) &entry;
+			data_datum.dsize = sizeof(ippool_info);
 
-				if (data->fd >= 0) rad_lockfd(data->fd, sizeof(int));
-				rcode = gdbm_store(data->gdbm, key_datum, data_datum, GDBM_REPLACE);
-				if (data->fd >= 0) rad_unlockfd(data->fd, sizeof(int));
-				if (rcode < 0) {
-					radlog(L_ERR, "rlm_ippool: Failed storing data to %s: %s",
-						data->session_db, gdbm_strerror(gdbm_errno));
-					return RLM_MODULE_FAIL;
-				}
-				/* Decrease allocated count from the ip index */
+			pthread_mutex_lock(&data->session_mutex);
+			rcode = gdbm_store(data->gdbm, key_datum, data_datum, GDBM_REPLACE);
+			pthread_mutex_unlock(&data->session_mutex);
+			if (rcode < 0) {
+				radlog(L_ERR, "rlm_ippool: Failed storing data to %s: %s",
+					data->session_db, gdbm_strerror(gdbm_errno));
+				return RLM_MODULE_FAIL;
+			}
+			/* Decrease allocated count from the ip index */
 
-				key_datum.dptr = (uint32_t *) &entry.ipaddr;
-				key_datum.dsize = sizeof(uint32_t);
-				if (data->ip_fd >= 0) rad_lockfd(data->ip_fd, sizeof(int));
-				data_datum = gdbm_fetch(data->ip, key_datum);
-				if (data->ip_fd >= 0) rad_unlockfd(data->ip_fd, sizeof(int));
-				if (data_datum.dptr != NULL){
-					memcpy(&num, data_datum.dptr, sizeof(int));
-					free(data_datum.dptr);
-					if (num >0){
-						num--;
-						DEBUG("rlm_ippool: num: %d",num);
-						data_datum.dptr = (int *) &num;
-						data_datum.dsize = sizeof(int);
-						if (data->ip_fd >= 0) rad_lockfd(data->ip_fd, sizeof(int));
-						rcode = gdbm_store(data->ip, key_datum, data_datum, GDBM_REPLACE);
-						if (data->ip_fd >= 0) rad_unlockfd(data->ip_fd, sizeof(int));
-						if (rcode < 0) {
-							radlog(L_ERR, "rlm_ippool: Failed storing data to %s: %s",
-									data->ip_index, gdbm_strerror(gdbm_errno));
-							return RLM_MODULE_FAIL;
-						}
+			key_datum.dptr = (uint32_t *) &entry.ipaddr;
+			key_datum.dsize = sizeof(uint32_t);
+			pthread_mutex_lock(&data->ip_mutex);
+			data_datum = gdbm_fetch(data->ip, key_datum);
+			pthread_mutex_unlock(&data->ip_mutex);
+			if (data_datum.dptr != NULL){
+				memcpy(&num, data_datum.dptr, sizeof(int));
+				free(data_datum.dptr);
+				if (num >0){
+					num--;
+					DEBUG("rlm_ippool: num: %d",num);
+					data_datum.dptr = (int *) &num;
+					data_datum.dsize = sizeof(int);
+					pthread_mutex_lock(&data->ip_mutex);
+					rcode = gdbm_store(data->ip, key_datum, data_datum, GDBM_REPLACE);
+					pthread_mutex_unlock(&data->ip_mutex);
+					if (rcode < 0) {
+						radlog(L_ERR, "rlm_ippool: Failed storing data to %s: %s",
+								data->ip_index, gdbm_strerror(gdbm_errno));
+						return RLM_MODULE_FAIL;
 					}
 				}
 			}
@@ -504,7 +512,7 @@ static int ippool_authorize(void *instance, REQUEST *request)
 	 * Walk through the database searching for an active=0 entry.
 	 */
 
-	if (data->fd >= 0) rad_lockfd(data->fd, sizeof(int));
+	pthread_mutex_lock(&data->session_mutex);
 	key_datum = gdbm_firstkey(data->gdbm);
 	while(key_datum.dptr){
 		data_datum = gdbm_fetch(data->gdbm, key_datum);
@@ -525,9 +533,9 @@ static int ippool_authorize(void *instance, REQUEST *request)
 
 				tmp.dptr = (uint32_t *) &entry.ipaddr;
 				tmp.dsize = sizeof(uint32_t);
-				if (data->ip_fd >= 0) rad_lockfd(data->ip_fd, sizeof(int));
+				pthread_mutex_lock(&data->ip_mutex);
 				data_datum = gdbm_fetch(data->ip, tmp);
-				if (data->ip_fd >= 0) rad_unlockfd(data->ip_fd, sizeof(int));
+				pthread_mutex_unlock(&data->ip_mutex);
 
 				/*
 				 * If we find an entry in the ip index and the number is zero (meaning
@@ -556,7 +564,7 @@ static int ippool_authorize(void *instance, REQUEST *request)
 		free(key_datum.dptr);
 		key_datum = nextkey;
 	}
-	if (data->fd >= 0) rad_unlockfd(data->fd, sizeof(int));
+	pthread_mutex_unlock(&data->session_mutex);
 	/*
 	 * If we have found a free entry set active to 1 then add a Framed-IP-Address attribute to
 	 * the reply
@@ -570,19 +578,21 @@ static int ippool_authorize(void *instance, REQUEST *request)
 			/*
 		 	 * Delete the entry so that we can change the key
 		 	 */
-			if (data->fd >= 0) rad_lockfd(data->fd, sizeof(int));
+			pthread_mutex_lock(&data->session_mutex);
 			gdbm_delete(data->gdbm, key_datum);
-			if (data->fd >= 0) rad_unlockfd(data->fd, sizeof(int));
+			pthread_mutex_unlock(&data->session_mutex);
 		}
 		free(key_datum.dptr);
+		memset(key.nas,0,MAX_NAS_NAME_SIZE);
 		strncpy(key.nas,nas,MAX_NAS_NAME_SIZE - 1);
 		key.port = port;
 		key_datum.dptr = (ippool_key *) &key;
 		key_datum.dsize = sizeof(ippool_key);
 		
-		if (data->fd >= 0) rad_lockfd(data->fd, sizeof(int));
+		DEBUG2("rlm_ippool: Allocating ip to nas/port: %s/%d",key.nas,key.port);
+		pthread_mutex_lock(&data->session_mutex);
 		rcode = gdbm_store(data->gdbm, key_datum, data_datum, GDBM_REPLACE);
-		if (data->fd >= 0) rad_unlockfd(data->fd, sizeof(int));
+		pthread_mutex_unlock(&data->session_mutex);
 		if (rcode < 0) {
 			radlog(L_ERR, "rlm_ippool: Failed storing data to %s: %s",
 				data->session_db, gdbm_strerror(gdbm_errno));
@@ -592,9 +602,9 @@ static int ippool_authorize(void *instance, REQUEST *request)
 		/* Increase the ip index count */
 		key_datum.dptr = (uint32_t *) &entry.ipaddr;
 		key_datum.dsize = sizeof(uint32_t);	
-		if (data->ip_fd >= 0) rad_lockfd(data->ip_fd, sizeof(int));
+		pthread_mutex_lock(&data->ip_mutex);
 		data_datum = gdbm_fetch(data->ip, key_datum);
-		if (data->ip_fd >= 0) rad_unlockfd(data->ip_fd, sizeof(int));
+		pthread_mutex_unlock(&data->ip_mutex);
 		if (data_datum.dptr){
 			memcpy(&num,data_datum.dptr,sizeof(int));
 			free(data_datum.dptr);
@@ -603,9 +613,9 @@ static int ippool_authorize(void *instance, REQUEST *request)
 		DEBUG("rlm_ippool: num: %d",num);
 		data_datum.dptr = (int *) &num;
 		data_datum.dsize = sizeof(int);
-		if (data->ip_fd >= 0) rad_lockfd(data->ip_fd, sizeof(int));
+		pthread_mutex_lock(&data->ip_mutex);
 		rcode = gdbm_store(data->ip, key_datum, data_datum, GDBM_REPLACE);
-		if (data->ip_fd >= 0) rad_unlockfd(data->ip_fd, sizeof(int));
+		pthread_mutex_unlock(&data->ip_mutex);
 		if (rcode < 0) {
 			radlog(L_ERR, "rlm_ippool: Failed storing data to %s: %s",
 				data->ip_index, gdbm_strerror(gdbm_errno));
@@ -638,6 +648,8 @@ static int ippool_detach(void *instance)
 	gdbm_close(data->ip);
 	free(data->session_db);
 	free(data->ip_index);
+	pthread_mutex_destroy(&data->session_mutex);
+	pthread_mutex_destroy(&data->ip_mutex);
 
 	free(instance);
 	return 0;
@@ -659,10 +671,13 @@ module_t rlm_ippool = {
 	ippool_instantiate,		/* instantiation */
 	{
 		NULL,			/* authentication */
-		ippool_authorize, 	/* authorization */
+		NULL,		 	/* authorization */
 		NULL,			/* preaccounting */
 		ippool_accounting,	/* accounting */
-		NULL			/* checksimul */
+		NULL,			/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		ippool_postauth		/* post-auth */
 	},
 	ippool_detach,			/* detach */
 	NULL,				/* destroy */
