@@ -1,7 +1,7 @@
 /*
  * rlm_pap.c
  *
- * Version:  $Id: rlm_pap.c,v 1.7 2002/07/11 19:35:46 aland Exp $
+ * Version:  $Id: rlm_pap.c,v 1.9 2002/10/22 13:49:09 kkalev Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@
 	free((char *)inst->scheme); \
 	free(inst)
 
-static const char rcsid[] = "$Id: rlm_pap.c,v 1.7 2002/07/11 19:35:46 aland Exp $";
+static const char rcsid[] = "$Id: rlm_pap.c,v 1.9 2002/10/22 13:49:09 kkalev Exp $";
 
 /*
  *      Define a structure for our module configuration.
@@ -284,7 +284,8 @@ static int pap_detach(void *instance)
 	rlm_pap_t *inst = (rlm_pap_t *) instance;
 
 #if HAVE_PTHREAD_H
-	pthread_mutex_destroy(&inst->mutex);
+	if (inst->sch == PAP_ENC_CRYPT)
+		pthread_mutex_destroy(&inst->mutex);
 #endif
 	PAP_INST_FREE(inst);
 	return 0;
@@ -310,7 +311,10 @@ module_t rlm_pap = {
 		NULL,		 	/* authorization */
 		NULL,			/* preaccounting */
 		NULL,			/* accounting */
-		NULL			/* checksimul */
+		NULL,			/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		NULL			/* post-auth */
 	},
 	pap_detach,			/* detach */
 	NULL,				/* destroy */
