@@ -33,7 +33,7 @@
 #include "modules.h"
 #include "conffile.h"
 
-static const char rcsid[] = "$Id: rlm_python.c,v 1.2 2002/07/09 19:16:24 aland Exp $";
+static const char rcsid[] = "$Id: rlm_python.c,v 1.3 2002/08/24 16:55:00 aland Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -934,7 +934,10 @@ module_t rlm_python = {
 		python_authorize,	/* authorization */
 		python_preacct,		/* preaccounting */
 		python_accounting,	/* accounting */
-		python_checksimul	/* checksimul */
+		python_checksimul,	/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		NULL			/* post-auth */
 	},
 	python_detach,			/* detach */
 	NULL,				/* destroy */
