@@ -55,8 +55,8 @@ CREATE TABLE radcheck (
   id int(11) unsigned NOT NULL auto_increment,
   UserName varchar(64) NOT NULL default '',
   Attribute varchar(32)  NOT NULL default '',
+  op char(2) NOT NULL,
   Value varchar(253) NOT NULL default '',
-  op char(2),
   PRIMARY KEY  (id),
   KEY UserName (UserName(32))
 ) ;
@@ -69,8 +69,8 @@ CREATE TABLE radgroupcheck (
   id int(11) unsigned NOT NULL auto_increment,
   GroupName varchar(64) NOT NULL default '',
   Attribute varchar(32)  NOT NULL default '',
+  op char(2) NOT NULL,
   Value varchar(253)  NOT NULL default '',
-  op char(2),
   PRIMARY KEY  (id),
   KEY GroupName (GroupName(32))
 ) ;
@@ -83,8 +83,8 @@ CREATE TABLE radgroupreply (
   id int(11) unsigned NOT NULL auto_increment,
   GroupName varchar(64) NOT NULL default '',
   Attribute varchar(32)  NOT NULL default '',
+  op char(2) NOT NULL,
   Value varchar(253)  NOT NULL default '',
-  op char(2),
   prio int unsigned NOT NULL default '0',
   PRIMARY KEY  (id),
   KEY GroupName (GroupName(32))
@@ -98,8 +98,8 @@ CREATE TABLE radreply (
   id int(11) unsigned NOT NULL auto_increment,
   UserName varchar(64) NOT NULL default '',
   Attribute varchar(32) NOT NULL default '',
+  op char(2) NOT NULL,
   Value varchar(253) NOT NULL default '',
-  op char(2),
   PRIMARY KEY  (id),
   KEY UserName (UserName(32))
 ) ;
