@@ -1,7 +1,7 @@
 /*
  * sql_mysql.c		SQL Module
  *
- * Version:	$Id: sql_mysql.c,v 1.4 2002/07/24 09:32:00 fcusack Exp $
+ * Version:	$Id: sql_mysql.c,v 1.10 2002/11/05 18:24:54 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -30,8 +30,16 @@
 #include <mysql/errmsg.h>
 
 #include 	"radiusd.h"
-#include	"sql_mysql.h"
 
+#include	<mysql/mysql.h>
+#include	"rlm_sql.h"
+
+typedef struct rlm_sql_mysql_sock {
+	MYSQL conn;
+	MYSQL *sock;
+	MYSQL_RES *result;
+	SQL_ROW row;
+} rlm_sql_mysql_sock;
 
 /*************************************************************************
  *
@@ -40,7 +48,7 @@
  *	Purpose: Establish connection to the db
  *
  *************************************************************************/
-int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+static int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 
 	rlm_sql_mysql_sock *mysql_sock;
 
@@ -48,14 +56,14 @@ int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 
 	mysql_sock = sqlsocket->conn;
 
-	radlog(L_INFO, "rlm_sql: Starting connect to MySQL server for #%d",
+	radlog(L_INFO, "rlm_sql_mysql: Starting connect to MySQL server for #%d",
 			sqlsocket->id);
 
 	mysql_init(&(mysql_sock->conn));
 	if (!(mysql_sock->sock = mysql_real_connect(&(mysql_sock->conn), config->sql_server, config->sql_login, config->sql_password,
 													config->sql_db, 0, NULL, CLIENT_FOUND_ROWS))) {
-		radlog(L_ERR, "rlm_sql: Couldn't connect socket to MySQL server %s@%s:%s", config->sql_login, config->sql_server, config->sql_db);
-		radlog(L_ERR, "rlm_sql:  Mysql error '%s'", mysql_error(&mysql_sock->conn));
+		radlog(L_ERR, "rlm_sql_mysql: Couldn't connect socket to MySQL server %s@%s:%s", config->sql_login, config->sql_server, config->sql_db);
+		radlog(L_ERR, "rlm_sql_mysql: Mysql error '%s'", mysql_error(&mysql_sock->conn));
 		mysql_sock->sock = NULL;
 		return -1;
 	}
@@ -72,7 +80,7 @@ int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
  *	Purpose: Free socket and any private connection data
  *
  *************************************************************************/
-int sql_destroy_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+static int sql_destroy_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 
 	rlm_sql_mysql_sock *mysql_sock = sqlsocket->conn;
 
@@ -83,6 +91,35 @@ int sql_destroy_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 }
 
 
+/*************************************************************************
+ *
+ *	Function: sql_check_error
+ *
+ *	Purpose: check the error to see if the server is down
+ *
+ *************************************************************************/
+static int sql_check_error(int error) {
+	switch(error) {
+	case CR_SERVER_GONE_ERROR:
+	case CR_SERVER_LOST:
+	case -1:
+		radlog(L_DBG, "rlm_sql_mysql: MYSQL check_error: %d, returning SQL_DOWN", error);
+		return SQL_DOWN;
+		break;
+	case 0:
+		return 0;
+		break;
+	case CR_OUT_OF_MEMORY:
+	case CR_COMMANDS_OUT_OF_SYNC:
+	case CR_UNKNOWN_ERROR:
+	default:
+		radlog(L_DBG, "rlm_sql_mysql: MYSQL check_error: %d received", error);
+		return -1;
+		break;
+	}
+}
+
+
 /*************************************************************************
  *
  *	Function: sql_query
@@ -90,14 +127,14 @@ int sql_destroy_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
  *	Purpose: Issue a query to the database
  *
  *************************************************************************/
-int sql_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
+static int sql_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
 
 	rlm_sql_mysql_sock *mysql_sock = sqlsocket->conn;
 
 	if (config->sqltrace)
-		radlog(L_DBG,"query:  %s", querystr);
+		radlog(L_DBG,"rlm_sql_mysql: query:  %s", querystr);
 	if (mysql_sock->sock == NULL) {
-		radlog(L_ERR, "Socket not connected");
+		radlog(L_ERR, "rlm_sql_mysql: Socket not connected");
 		return SQL_DOWN;
 	}
 
@@ -106,6 +143,57 @@ int sql_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
 }
 
 
+/*************************************************************************
+ *
+ *	Function: sql_store_result
+ *
+ *	Purpose: database specific store_result function. Returns a result
+ *               set for the query.
+ *
+ *************************************************************************/
+static int sql_store_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+
+	rlm_sql_mysql_sock *mysql_sock = sqlsocket->conn;
+
+	if (mysql_sock->sock == NULL) {
+		radlog(L_ERR, "rlm_sql_mysql: Socket not connected");
+		return SQL_DOWN;
+	}
+	if (!(mysql_sock->result = mysql_store_result(mysql_sock->sock))) {
+		radlog(L_ERR, "rlm_sql_mysql: MYSQL Error: Cannot get result");
+		radlog(L_ERR, "rlm_sql_mysql: MYSQL Error: %s", mysql_error(mysql_sock->sock));
+		return sql_check_error(mysql_errno(mysql_sock->sock));
+	}
+	return 0;
+}
+
+
+/*************************************************************************
+ *
+ *	Function: sql_num_fields
+ *
+ *	Purpose: database specific num_fields function. Returns number
+ *               of columns from query
+ *
+ *************************************************************************/
+static int sql_num_fields(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+
+
+	int     num = 0;
+	rlm_sql_mysql_sock *mysql_sock = sqlsocket->conn;
+
+#if MYSQL_VERSION_ID >= 32224
+	if (!(num = mysql_field_count(mysql_sock->sock))) {
+#else
+	if (!(num = mysql_num_fields(mysql_sock->sock))) {
+#endif
+		radlog(L_ERR, "rlm_sql_mysql: MYSQL Error: No Fields");
+		radlog(L_ERR, "rlm_sql_mysql: MYSQL error: %s", mysql_error(mysql_sock->sock));
+	}
+	return num;
+}
+
+
 /*************************************************************************
  *
  *	Function: sql_select_query
@@ -113,7 +201,7 @@ int sql_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
  *	Purpose: Issue a select query to the database
  *
  *************************************************************************/
-int sql_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
+static int sql_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
 
 	int ret;
 
@@ -135,57 +223,6 @@ int sql_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
 }
 
 
-/*************************************************************************
- *
- *	Function: sql_store_result
- *
- *	Purpose: database specific store_result function. Returns a result
- *               set for the query.
- *
- *************************************************************************/
-int sql_store_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
-
-	rlm_sql_mysql_sock *mysql_sock = sqlsocket->conn;
-
-	if (mysql_sock->sock == NULL) {
-		radlog(L_ERR, "Socket not connected");
-		return SQL_DOWN;
-	}
-	if (!(mysql_sock->result = mysql_store_result(mysql_sock->sock))) {
-		radlog(L_ERR, "MYSQL Error: Cannot get result");
-		radlog(L_ERR, "MYSQL Error: %s", mysql_error(mysql_sock->sock));
-		return sql_check_error(mysql_errno(mysql_sock->sock));
-	}
-	return 0;
-}
-
-
-/*************************************************************************
- *
- *	Function: sql_num_fields
- *
- *	Purpose: database specific num_fields function. Returns number
- *               of columns from query
- *
- *************************************************************************/
-int sql_num_fields(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
-
-
-	int     num = 0;
-	rlm_sql_mysql_sock *mysql_sock = sqlsocket->conn;
-
-#if MYSQL_VERSION_ID >= 32224
-	if (!(num = mysql_field_count(mysql_sock->sock))) {
-#else
-	if (!(num = mysql_num_fields(mysql_sock->sock))) {
-#endif
-		radlog(L_ERR, "MYSQL Error: No Fields");
-		radlog(L_ERR, "MYSQL error: %s", mysql_error(mysql_sock->sock));
-	}
-	return num;
-}
-
-
 /*************************************************************************
  *
  *	Function: sql_num_rows
@@ -194,7 +231,7 @@ int sql_num_fields(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *               query
  *
  *************************************************************************/
-int sql_num_rows(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_num_rows(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	rlm_sql_mysql_sock *mysql_sock = sqlsocket->conn;
 
@@ -214,7 +251,7 @@ int sql_num_rows(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *		 0 on success, -1 on failure, SQL_DOWN if database is down.
  *
  *************************************************************************/
-int sql_fetch_row(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_fetch_row(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	rlm_sql_mysql_sock *mysql_sock = sqlsocket->conn;
 
@@ -227,36 +264,6 @@ int sql_fetch_row(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 }
 
 
-/*************************************************************************
- *
- *	Function: sql_check_error
- *
- *	Purpose: check the error to see if the server is down
- *
- *************************************************************************/
-int sql_check_error(int error) {
-	switch(error) {
-	case CR_SERVER_GONE_ERROR:
-	case CR_SERVER_LOST:
-	case -1:
-		radlog(L_DBG, "MYSQL check_error: %d, returning SQL_DOWN", error);
-		return SQL_DOWN;
-		break;
-	case 0:
-		return 0;
-		break;
-	case CR_OUT_OF_MEMORY:
-	case CR_COMMANDS_OUT_OF_SYNC:
-	case CR_UNKNOWN_ERROR:
-	default:
-		radlog(L_DBG, "MYSQL check_error: %d received", error);
-		return -1;
-		break;
-	}
-}
-
-
-
 /*************************************************************************
  *
  *	Function: sql_free_result
@@ -265,7 +272,7 @@ int sql_check_error(int error) {
  *               for a result set
  *
  *************************************************************************/
-int sql_free_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_free_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	rlm_sql_mysql_sock *mysql_sock = sqlsocket->conn;
 
@@ -286,10 +293,13 @@ int sql_free_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *               connection
  *
  *************************************************************************/
-char *sql_error(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static char *sql_error(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	rlm_sql_mysql_sock *mysql_sock = sqlsocket->conn;
 
+	if (mysql_sock == NULL || mysql_sock->sock == NULL) {
+		return "rlm_sql_mysql: no connection to db";
+	}
 	return mysql_error(mysql_sock->sock);
 }
 
@@ -302,12 +312,14 @@ char *sql_error(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *               connection
  *
  *************************************************************************/
-int sql_close(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_close(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	rlm_sql_mysql_sock *mysql_sock = sqlsocket->conn;
 
-	mysql_close(mysql_sock->sock);
-	mysql_sock->sock = NULL;
+	if (mysql_sock && mysql_sock->sock){
+		mysql_close(mysql_sock->sock);
+		mysql_sock->sock = NULL;
+	}
 
 	return 0;
 }
@@ -320,7 +332,7 @@ int sql_close(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *	Purpose: End the query, such as freeing memory
  *
  *************************************************************************/
-int sql_finish_query(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_finish_query(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	return 0;
 }
@@ -334,7 +346,7 @@ int sql_finish_query(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *	Purpose: End the select query, such as freeing memory or result
  *
  *************************************************************************/
-int sql_finish_select_query(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_finish_select_query(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	sql_free_result(sqlsocket, config);
 
@@ -349,7 +361,7 @@ int sql_finish_select_query(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *	Purpose: End the select query, such as freeing memory or result
  *
  *************************************************************************/
-int sql_affected_rows(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_affected_rows(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	rlm_sql_mysql_sock *mysql_sock = sqlsocket->conn;
 
