@@ -11,12 +11,50 @@
 #include <string.h>
 
 #include 	"radiusd.h"
-#include	"sql_oracle.h"
+
+#include <oci.h>
+#include "rlm_sql.h"
+
+typedef struct rlm_sql_oracle_sock {
+	OCIEnv		*env;
+	OCIError	*errHandle;
+	OCISvcCtx	*conn;
+	OCIStmt		*queryHandle;
+	char		**results;
+	int		id;
+	int		in_use;
+	struct timeval	tv;
+} rlm_sql_oracle_sock;
 
 #define	MAX_DATASTR_LEN	64
 
 
-sb2 indicators[5];
+/*************************************************************************
+ *
+ *	Function: sql_error
+ *
+ *	Purpose: database specific error. Returns error associated with
+ *               connection
+ *
+ *************************************************************************/
+static char *sql_error(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+
+	static char	msgbuf[512];
+	sb4		errcode = 0;
+	rlm_sql_oracle_sock *oracle_sock = sqlsocket->conn;
+ 
+	memset((void *) msgbuf, (int)'\0', sizeof(msgbuf));
+
+	OCIErrorGet((dvoid *) oracle_sock->errHandle, (ub4) 1, (text *) NULL,
+		&errcode, msgbuf, (ub4) sizeof(msgbuf), (ub4) OCI_HTYPE_ERROR);
+	if (errcode) {
+		return msgbuf;
+	}
+	else {
+		return NULL;
+	}
+}
+
 
 /*************************************************************************
  *
@@ -25,7 +63,7 @@ sb2 indicators[5];
  *	Purpose: Establish connection to the db
  *
  *************************************************************************/
-int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+static int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 	
 	rlm_sql_oracle_sock *oracle_sock;
 
@@ -38,14 +76,14 @@ int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 		(dvoid * (*)(dvoid *, dvoid *, size_t))0, 
 		(void (*)(dvoid *, dvoid *)) 0,
 		0, (dvoid **)0 )) {
-		radlog(L_ERR,"Init: Couldn't init Oracle OCI environment (OCIEnvCreate())");
+		radlog(L_ERR,"rlm_sql_oracle: Couldn't init Oracle OCI environment (OCIEnvCreate())");
 		return -1;
 	}
 
 	if (OCIHandleAlloc((dvoid *) oracle_sock->env, (dvoid **) &oracle_sock->errHandle,
 		(ub4) OCI_HTYPE_ERROR, (size_t) 0, (dvoid **) 0))
 	{
-		radlog(L_ERR,"Init: Couldn't init Oracle ERROR handle (OCIHandleAlloc())");
+		radlog(L_ERR,"rlm_sql_oracle: Couldn't init Oracle ERROR handle (OCIHandleAlloc())");
 		return -1;
 	}
 
@@ -55,7 +93,7 @@ int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 	    ||  OCIHandleAlloc((dvoid *)oracle_sock->env, (dvoid **) &oracle_sock->queryHandle,
 				(ub4)OCI_HTYPE_STMT, (CONST size_t) 0, (dvoid **) 0))
 	{
-		radlog(L_ERR,"Init: Couldn't init Oracle query handles: %s",
+		radlog(L_ERR,"rlm_sql_oracle: Couldn't init Oracle query handles: %s",
 			sql_error(sqlsocket, config));
 		return -1;
 	}
@@ -66,7 +104,7 @@ int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 			config->sql_password,  strlen(config->sql_password),
 			config->sql_db, strlen(config->sql_db)))
 	{
-		radlog(L_ERR,"Init: Oracle logon failed: '%s'", sql_error(sqlsocket, config));
+		radlog(L_ERR,"rlm_sql_oracle: Oracle logon failed: '%s'", sql_error(sqlsocket, config));
 		return -1;
 	}
 
@@ -81,7 +119,7 @@ int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
  *	Purpose: Free socket and private connection data
  *
  *************************************************************************/
-int sql_destroy_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+static int sql_destroy_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 
 	/* FIXME: Someone write the oracle specific disconnect and free code!! */
 
@@ -89,6 +127,33 @@ int sql_destroy_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 	return 0;
 }
 
+/*************************************************************************
+ *
+ *	Function: sql_num_fields
+ *
+ *	Purpose: database specific num_fields function. Returns number
+ *               of columns from query
+ *
+ *************************************************************************/
+static int sql_num_fields(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+
+	ub4		count;
+	rlm_sql_oracle_sock *oracle_sock = sqlsocket->conn;
+
+	/* get the number of columns in the select list */ 
+	if (OCIAttrGet ((dvoid *)oracle_sock->queryHandle,
+			(ub4)OCI_HTYPE_STMT,
+			(dvoid *) &count,
+			(ub4 *) 0,
+			(ub4)OCI_ATTR_PARAM_COUNT,
+			oracle_sock->errHandle)) {
+		radlog(L_ERR,"rlm_sql_oracle: Error retrieving column count in sql_num_fields: %s",
+			sql_error(sqlsocket, config));
+		return -1;
+	}
+	return count;
+}
+
 /*************************************************************************
  *
  *	Function: sql_query
@@ -97,22 +162,22 @@ int sql_destroy_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
  *               the database.
  *
  *************************************************************************/
-int sql_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
+static int sql_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
 
 	int	x;
 	rlm_sql_oracle_sock *oracle_sock = sqlsocket->conn;
 
 	if (config->sqltrace)
 		DEBUG(querystr);
-	 if (oracle_sock->conn == NULL) {
+	if (oracle_sock->conn == NULL) {
 		radlog(L_ERR, "Socket not connected");
-		return 0;
+		return SQL_DOWN;
 	}
 
 	if (OCIStmtPrepare (oracle_sock->queryHandle, oracle_sock->errHandle,
 				querystr, strlen(querystr),
 				OCI_NTV_SYNTAX, OCI_DEFAULT))  {
-		radlog(L_ERR,"sql_query: prepare failed: %s",sql_error(sqlsocket, config));
+		radlog(L_ERR,"rlm_sql_oracle: prepare failed in sql_query: %s",sql_error(sqlsocket, config));
 		return -1;
 	}
 
@@ -126,12 +191,12 @@ int sql_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
 				(ub4) OCI_DEFAULT);
 
 	if ((x != OCI_NO_DATA) && (x != OCI_SUCCESS)) {
-		return -1;
+		return SQL_DOWN;
 	}
 
 	x = OCITransCommit(oracle_sock->conn, oracle_sock->errHandle, (ub4) 0);
 	if (x != OCI_SUCCESS)
-		return -1;
+		return SQL_DOWN;
 
 	return 0;
 }
@@ -144,7 +209,7 @@ int sql_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
  *	Purpose: Issue a select query to the database
  *
  *************************************************************************/
-int sql_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
+static int sql_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
 
 	int		x;
 	int		y;
@@ -158,15 +223,15 @@ int sql_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
 
 	if (config->sqltrace)
 		DEBUG(querystr);
-	 if (oracle_sock->conn == NULL) {
-		radlog(L_ERR, "Socket not connected");
-		return -1;
+	if (oracle_sock->conn == NULL) {
+		radlog(L_ERR, "rlm_sql_oracle: Socket not connected");
+		return SQL_DOWN;
 	}
 
 	if (OCIStmtPrepare (oracle_sock->queryHandle, oracle_sock->errHandle,
 				querystr, strlen(querystr),
 				OCI_NTV_SYNTAX, OCI_DEFAULT))  {
-		radlog(L_ERR,"sql_select_query: prepare failed: %s",sql_error(sqlsocket, config));
+		radlog(L_ERR,"rlm_sql_oracle: prepare failed in sql_select_query: %s",sql_error(sqlsocket, config));
 		return -1;
 	}
 
@@ -185,7 +250,7 @@ int sql_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
 		return 0;
 	}
 	else if (x != OCI_SUCCESS) {
-		return -1;
+		return SQL_DOWN;
 	}
 
 	/*
@@ -203,12 +268,14 @@ int sql_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
 	memset(rowdata, 0, (sizeof(char *) * (colcount+1) ));
 
 	for (y=1; y <= colcount; y++) {
+		sb2 indicators[5];
+
 		x=OCIParamGet(oracle_sock->queryHandle, OCI_HTYPE_STMT,
 				oracle_sock->errHandle,
 				(dvoid **)&param,
 				(ub4) y);
 		if (x != OCI_SUCCESS) {
-			radlog(L_ERR,"sql_select_query: OCIParamGet() failed: %s",
+			radlog(L_ERR,"rlm_sql_oracle: OCIParamGet() failed in sql_select_query: %s",
 				sql_error(sqlsocket, config));
 			return -1;
 		}
@@ -217,7 +284,7 @@ int sql_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
 			   (dvoid*)&dtype, (ub4*)0, OCI_ATTR_DATA_TYPE,
 			   oracle_sock->errHandle);
 		if (x != OCI_SUCCESS) {
-			radlog(L_ERR,"sql_select_query: OCIAttrGet() failed: %s",
+			radlog(L_ERR,"rlm_sql_oracle: OCIAttrGet() failed in sql_select_query: %s",
 				sql_error(sqlsocket, config));
 			return -1;
 		}
@@ -236,7 +303,7 @@ int sql_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
 				   (dvoid*) &dsize, (ub4 *)0, (ub4) OCI_ATTR_DATA_SIZE,
 				   oracle_sock->errHandle);
 			if (x != OCI_SUCCESS) {
-				radlog(L_ERR,"sql_select_query: OCIAttrGet() failed: %s",
+				radlog(L_ERR,"rlm_sql_oracle: OCIAttrGet() failed in sql_select_query: %s",
 					sql_error(sqlsocket, config));
 				return -1;
 			}
@@ -270,7 +337,7 @@ int sql_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
 				(dvoid *) 0,
 				OCI_DEFAULT);
 		if (x != OCI_SUCCESS) {
-			radlog(L_ERR,"sql_select_query: OCIDefineByPos() failed: %s",
+			radlog(L_ERR,"rlm_sql_oracle: OCIDefineByPos() failed in sql_select_query: %s",
 				sql_error(sqlsocket, config));
 			return -1;
 		}
@@ -280,7 +347,7 @@ int sql_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
 
 	oracle_sock->results=rowdata;
 
-	return 1;
+	return 0;
 }
 
 
@@ -292,39 +359,12 @@ int sql_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config, char *querystr) {
  *               set for the query.
  *
  *************************************************************************/
-int sql_store_result(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+static int sql_store_result(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 	/* Not needed for Oracle */
 	return 0;
 }
 
 
-/*************************************************************************
- *
- *	Function: sql_num_fields
- *
- *	Purpose: database specific num_fields function. Returns number
- *               of columns from query
- *
- *************************************************************************/
-int sql_num_fields(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
-
-	ub4		count;
-	rlm_sql_oracle_sock *oracle_sock = sqlsocket->conn;
-
-	/* get the number of columns in the select list */ 
-	if (OCIAttrGet ((dvoid *)oracle_sock->queryHandle,
-			(ub4)OCI_HTYPE_STMT,
-			(dvoid *) &count,
-			(ub4 *) 0,
-			(ub4)OCI_ATTR_PARAM_COUNT,
-			oracle_sock->errHandle)) {
-		radlog(L_ERR,"sql_num_fields: error retrieving colun count: %s",
-			sql_error(sqlsocket, config));
-		return -1;
-	}
-	return count;
-}
-
 /*************************************************************************
  *
  *	Function: sql_num_rows
@@ -333,7 +373,7 @@ int sql_num_fields(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
  *               query
  *
  *************************************************************************/
-int sql_num_rows(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+static int sql_num_rows(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 
 	ub4	rows=0;
 	rlm_sql_oracle_sock *oracle_sock = sqlsocket->conn;
@@ -358,11 +398,16 @@ int sql_num_rows(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
  *		 0 on success, -1 on failure, SQL_DOWN if database is down.
  *
  *************************************************************************/
-int sql_fetch_row(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+static int sql_fetch_row(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 
 	int	x;
 	rlm_sql_oracle_sock *oracle_sock = sqlsocket->conn;
 
+	if (oracle_sock->conn == NULL) {
+		radlog(L_ERR, "rlm_sql_oracle: Socket not connected");
+		return SQL_DOWN;
+	}
+
 	sqlsocket->row = NULL;
 
 	x=OCIStmtFetch(oracle_sock->queryHandle,
@@ -371,13 +416,13 @@ int sql_fetch_row(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 			OCI_FETCH_NEXT,
 			OCI_DEFAULT);
 	if (x == OCI_NO_DATA) {
-		return NULL;
+		return -1;
 	}
 	else if (x != OCI_SUCCESS) {
 		/* XXX Check if x suggests we should return SQL_DOWN */
-		radlog(L_ERR,"sql_fetch_row: fetch failed: %s",
+		radlog(L_ERR,"rlm_sql_oracle: fetch failed in sql_fetch_row: %s",
 				sql_error(sqlsocket, config));
-		return -1;
+		return SQL_DOWN;
 	}
 
 	sqlsocket->row = oracle_sock->results;
@@ -394,7 +439,7 @@ int sql_fetch_row(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
  *               for a result set
  *
  *************************************************************************/
-int sql_free_result(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+static int sql_free_result(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 
 	int i=0;
 	int num_fields;
@@ -414,33 +459,6 @@ int sql_free_result(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 
 
 
-/*************************************************************************
- *
- *	Function: sql_error
- *
- *	Purpose: database specific error. Returns error associated with
- *               connection
- *
- *************************************************************************/
-char *sql_error(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
-
-	static char	msgbuf[512];
-	int		errcode = 0;
-	rlm_sql_oracle_sock *oracle_sock = sqlsocket->conn;
- 
-	memset((void *) msgbuf, (int)'\0', sizeof(msgbuf));
-
-	OCIErrorGet((dvoid *) oracle_sock->errHandle, (ub4) 1, (text *) NULL,
-		&errcode, msgbuf, (ub4) sizeof(msgbuf), (ub4) OCI_HTYPE_ERROR);
-	if (errcode) {
-		return msgbuf;
-	}
-	else {
-		return NULL;
-	}
-}
-
-
 /*************************************************************************
  *
  *	Function: sql_close
@@ -449,7 +467,7 @@ char *sql_error(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
  *               connection and cleans up any open handles.
  *
  *************************************************************************/
-int sql_close(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+static int sql_close(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 
 	rlm_sql_oracle_sock *oracle_sock = sqlsocket->conn;
 
@@ -480,7 +498,7 @@ int sql_close(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
  *	Purpose: End the query, such as freeing memory
  *
  *************************************************************************/
-int sql_finish_query(SQLSOCK *sqlsocket, SQL_CONFIG *config)
+static int sql_finish_query(SQLSOCK *sqlsocket, SQL_CONFIG *config)
 {
 	return 0;
 }
@@ -494,7 +512,7 @@ int sql_finish_query(SQLSOCK *sqlsocket, SQL_CONFIG *config)
  *	Purpose: End the select query, such as freeing memory or result
  *
  *************************************************************************/
-int sql_finish_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+static int sql_finish_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 
 	int 	x=0;
 	rlm_sql_oracle_sock *oracle_sock = sqlsocket->conn;
@@ -517,7 +535,7 @@ int sql_finish_select_query(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
  *               or insert)
  *
  *************************************************************************/
-int sql_affected_rows(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+static int sql_affected_rows(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 
 	return sql_num_rows(sqlsocket, config);
 }
