@@ -1,7 +1,7 @@
 /*
  * sql_postgresql.c		Postgresql rlm_sql driver
  *
- * Version:	$Id: sql_postgresql.c,v 1.23 2002/07/11 16:29:00 cparker Exp $
+ * Version:	$Id: sql_postgresql.c,v 1.26 2002/11/12 20:15:25 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -45,7 +45,19 @@
 #include <string.h>
 
 #include "radiusd.h"
-#include "sql_postgresql.h"
+
+#include        <libpq-fe.h>
+#include	"rlm_sql.h"
+
+typedef struct rlm_sql_postgres_sock {
+   PGconn          *conn;
+   PGresult        *result;
+   int             cur_row;
+   int             num_fields;
+    int		   affected_rows;
+   char            **row;
+} rlm_sql_postgres_sock;
+
 
 /* Internal function. Return true if the postgresql status value
  * indicates successful completion of the query. Return false otherwise
@@ -83,6 +95,45 @@ free_result_row(rlm_sql_postgres_sock * pg_sock)
 	}
 }
 
+/*************************************************************************
+ *
+ *      Function: sql_check_error
+ *
+ *      Purpose: check the error to see if the server is down
+ *
+ *	Note: It is possible that something other than a connection error
+ *	could cause PGRES_FATAL_ERROR. If that happens a reconnect will
+ *	occur anyway. Not optimal, but I couldn't find a way to check it.
+ *		 		Peter Nixon <codemonkey@peternixon.net>
+ *
+
+************************************************************************/
+static int sql_check_error(int error) {
+        switch(error) {
+        case PGRES_FATAL_ERROR:
+        case -1:
+                radlog(L_DBG, "rlm_sql_postgresql: Postgresql check_error:
+s, returning SQL_DOWN", PQresStatus(error));
+                return SQL_DOWN;
+                break;
+
+        case PGRES_COMMAND_OK:
+        case PGRES_TUPLES_OK:
+        case 0:
+                return 0;
+                break;
+
+        case PGRES_NONFATAL_ERROR:
+        case PGRES_BAD_RESPONSE:
+        default:
+                radlog(L_DBG, "rlm_sql_postgresql: Postgresql check_error:
+s received", PQresStatus(error));
+                return -1;
+                break;
+        }
+}
+
+
 /*************************************************************************
  *
  *	Function: sql_create_socket
@@ -90,7 +141,7 @@ free_result_row(rlm_sql_postgres_sock * pg_sock)
  *	Purpose: Establish connection to the db
  *
  *************************************************************************/
-int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
+static int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 	char connstring[2048];
 	char *port, *host;
 	rlm_sql_postgres_sock *pg_sock;
@@ -121,10 +172,10 @@ int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 	pg_sock->conn=PQconnectdb(connstring);
 
 	if (PQstatus(sqlsocket->conn) == CONNECTION_BAD) {
-		radlog(L_ERR, "sql_postgresql: Couldn't connect socket to PostgreSQL server %s@%s:%s", config->sql_login, config->sql_server, config->sql_db);
-		radlog(L_ERR, "sql_postgresql: Postgresql error '%s'", PQerrorMessage(pg_sock->conn));
+		radlog(L_ERR, "rlm_sql_postgresql: Couldn't connect socket to PostgreSQL server %s@%s:%s", config->sql_login, config->sql_server, config->sql_db);
+		radlog(L_ERR, "rlm_sql_postgresql: Postgresql error '%s'", PQerrorMessage(pg_sock->conn));
 		PQfinish(pg_sock->conn);
-		return -1;
+		return SQL_DOWN;
 	}
 
 	return 0;
@@ -137,34 +188,34 @@ int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
  *	Purpose: Issue a query to the database
  *
  *************************************************************************/
-int sql_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
+static int sql_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
 
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
 
 	if (config->sqltrace)
-		radlog(L_DBG,"query:\n%s", querystr);
+		radlog(L_DBG,"rlm_sql_postgresql: query:\n%s", querystr);
 
 	if (pg_sock->conn == NULL) {
-		radlog(L_ERR, "Socket not connected");
-		return -1;
+		radlog(L_ERR, "rlm_sql_postgresql: Socket not connected");
+		return SQL_DOWN;
 	}
 
 	pg_sock->result = PQexec(pg_sock->conn, querystr);
 	if (!pg_sock->result)
 	{
-		radlog(L_ERR, "PostgreSQL Query failed Error: %s", 
+		radlog(L_ERR, "rlm_sql_postgresql: PostgreSQL Query failed Error: %s", 
 				PQerrorMessage(pg_sock->conn));
 		return  -1;
 	} else {
 		ExecStatusType status = PQresultStatus(pg_sock->result);
 
-		radlog(L_DBG, "rlm_postgresql Status: %s", PQresStatus(status));
+		radlog(L_DBG, "rlm_sql_postgresql: Status: %s", PQresStatus(status));
 
-		radlog(L_DBG, "sql_postgresql: affected rows = %s",
+		radlog(L_DBG, "rlm_sql_postgresql: affected rows = %s",
 				PQcmdTuples(pg_sock->result));
 
 		if (!status_is_ok(status))
-			return -1;
+			return sql_check_error(status);
 
 		if (strncasecmp("select", querystr, 6) != 0) {
 			/* store the number of affected rows because the sql module
@@ -190,7 +241,7 @@ int sql_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
  *	Purpose: Issue a select query to the database
  *
  *************************************************************************/
-int sql_select_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
+static int sql_select_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
 	return sql_query(sqlsocket, config, querystr);
 }
 
@@ -203,7 +254,7 @@ int sql_select_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
  *               set for the query.
  *
  *************************************************************************/
-int sql_store_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_store_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
 
 	pg_sock->cur_row = 0;
@@ -220,14 +271,14 @@ int sql_store_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *               of columns from query
  *
  *************************************************************************/
-int sql_num_fields(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_num_fields(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	int num = 0;
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
 
 	if (!(num = PQnfields(pg_sock->result))) {
-		radlog(L_ERR, "PostgreSQL Error: Cannot get result");
-		radlog(L_ERR, "PostgreSQL error: %s", PQerrorMessage(pg_sock->conn));
+		radlog(L_ERR, "rlm_sql_postgresql: PostgreSQL Error: Cannot get result");
+		radlog(L_ERR, "rlm_sql_postgresql: PostgreSQL error: %s", PQerrorMessage(pg_sock->conn));
 	}
 	return num;
 }
@@ -242,7 +293,7 @@ int sql_num_fields(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *		 0 on success, -1 on failure, SQL_DOWN if 'database is down'.
  *
  *************************************************************************/
-int sql_fetch_row(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_fetch_row(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	int records, i, len;
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
@@ -285,7 +336,7 @@ int sql_fetch_row(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *               for a result set
  *
  *************************************************************************/
-int sql_free_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_free_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
 
@@ -313,7 +364,7 @@ int sql_free_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *               connection
  *
  *************************************************************************/
-char *sql_error(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static char *sql_error(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
 
@@ -329,10 +380,12 @@ char *sql_error(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *               connection
  *
  *************************************************************************/
-int sql_close(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_close(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
 
+	if (!pg_sock->conn) return 0;
+
 	PQfinish(pg_sock->conn);
 	pg_sock->conn = NULL;
 
@@ -347,7 +400,7 @@ int sql_close(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *	Purpose: End the query, such as freeing memory
  *
  *************************************************************************/
-int sql_finish_query(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_finish_query(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	return sql_free_result(sqlsocket, config);
 }
@@ -361,7 +414,7 @@ int sql_finish_query(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *	Purpose: End the select query, such as freeing memory or result
  *
  *************************************************************************/
-int sql_finish_select_query(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_finish_select_query(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	return sql_free_result(sqlsocket, config);
 }
@@ -374,7 +427,7 @@ int sql_finish_select_query(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
  *	Purpose: Return the number of rows affected by the last query.
  *
  *************************************************************************/
-int sql_affected_rows(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
+static int sql_affected_rows(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 	rlm_sql_postgres_sock *pg_sock = sqlsocket->conn;
 
 	return pg_sock->affected_rows;
