@@ -2,7 +2,7 @@
  * rlm_sql.c		SQL Module
  * 		Main SQL module file. Most ICRADIUS code is located in sql.c
  *
- * Version:	$Id: rlm_sql.c,v 1.92 2002/07/25 15:54:23 aland Exp $
+ * Version:	$Id: rlm_sql.c,v 1.108 2002/11/11 20:09:07 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-	"$Id: rlm_sql.c,v 1.92 2002/07/25 15:54:23 aland Exp $";
+	"$Id: rlm_sql.c,v 1.108 2002/11/11 20:09:07 aland Exp $";
 
 #include "autoconf.h"
 
@@ -69,21 +69,22 @@ static CONF_PARSER module_config[] = {
 	{"deletestalesessions", PW_TYPE_BOOLEAN, offsetof(SQL_CONFIG,deletestalesessions), NULL, "0"},
 	{"num_sql_socks", PW_TYPE_INTEGER, offsetof(SQL_CONFIG,num_sql_socks), NULL, "5"},
 	{"sql_user_name", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,query_user), NULL, ""},
+	{"default_user_profile", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,default_profile), NULL, ""},
+	{"query_on_not_found", PW_TYPE_BOOLEAN, offsetof(SQL_CONFIG,query_on_not_found), NULL, "no"},
 	{"authorize_check_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,authorize_check_query), NULL, ""},
 	{"authorize_reply_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,authorize_reply_query), NULL, ""},
 	{"authorize_group_check_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,authorize_group_check_query), NULL, ""},
 	{"authorize_group_reply_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,authorize_group_reply_query), NULL, ""},
-	{"authenticate_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,authenticate_query), NULL, ""},
 	{"accounting_onoff_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_onoff_query), NULL, ""},
 	{"accounting_update_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_update_query), NULL, ""},
 	{"accounting_start_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_start_query), NULL, ""},
 	{"accounting_start_query_alt", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_start_query_alt), NULL, ""},
 	{"accounting_stop_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_stop_query), NULL, ""},
 	{"accounting_stop_query_alt", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_stop_query_alt), NULL, ""},
+	{"group_membership_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,groupmemb_query), NULL, ""},
 	{"connect_failure_retry_delay", PW_TYPE_INTEGER, offsetof(SQL_CONFIG,connect_failure_retry_delay), NULL, "60"},
 	{"simul_count_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,simul_count_query), NULL, ""},
 	{"simul_verify_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,simul_verify_query), NULL, ""},
-	{"simul_zap_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,simul_zap_query), NULL, ""},
 
 	{NULL, -1, 0, NULL, NULL}
 };
@@ -112,16 +113,18 @@ static int sql_xlat(void *instance, REQUEST *request, char *fmt, char *out, int
 {
 	SQLSOCK *sqlsocket;
 	SQL_ROW row;
-	SQL_INST *inst=instance;
+	SQL_INST *inst = instance;
 	char querystr[MAX_QUERY_LEN];
 	int ret = 0;
 
-	DEBUG("rlm_sql: - sql_xlat");
+	DEBUG("rlm_sql (%s): - sql_xlat", inst->config->xlat_name);
+
 	/*
 	 * Do an xlat on the provided string (nice recursive operation).
 	 */
-	if (!radius_xlat(querystr, sizeof(querystr), fmt, request, func)){
-		radlog(L_ERR, "rlm_sql: xlat failed.");
+	if (!radius_xlat(querystr, sizeof(querystr), fmt, request, func)) {
+		radlog(L_ERR, "rlm_sql (%s): xlat failed.",
+		       inst->config->xlat_name);
 		return 0;
 	}
 
@@ -129,43 +132,53 @@ static int sql_xlat(void *instance, REQUEST *request, char *fmt, char *out, int
 	if (sqlsocket == NULL)
 		return 0;
 	if (rlm_sql_select_query(sqlsocket,inst,querystr)){
-		radlog(L_ERR, "rlm_sql: database query error");
+		radlog(L_ERR, "rlm_sql (%s): database query error",
+		       inst->config->xlat_name);
 		sql_release_socket(inst,sqlsocket);
 		return 0;
 	}
 
 	ret = rlm_sql_fetch_row(sqlsocket, inst);
-	(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
 
 	if (ret) {
-		DEBUG("rlm_sql: SQL query did not succeed");
+		DEBUG("rlm_sql (%s): SQL query did not succeed",
+		      inst->config->xlat_name);
+		(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
 		sql_release_socket(inst,sqlsocket);
 		return 0;
 	}
 
 	row = sqlsocket->row;
 	if (row == NULL) {
-		DEBUG("rlm_sql: SQL query did not return any results");
+		DEBUG("rlm_sql (%s): SQL query did not return any results",
+		      inst->config->xlat_name);
+		(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
 		sql_release_socket(inst,sqlsocket);
 		return 0;
 	}
 
 	if (row[0] == NULL){
-		DEBUG("rlm_sql: row[0] returned NULL");
+		DEBUG("rlm_sql (%s): row[0] returned NULL",
+		      inst->config->xlat_name);
+		(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
 		sql_release_socket(inst,sqlsocket);
 		return 0;
 	}
 	ret = strlen(row[0]);
 	if (ret > freespace){
-		DEBUG("rlm_sql: sql_xlat:: Insufficient string space");
+		DEBUG("rlm_sql (%s): sql_xlat:: Insufficient string space",
+		      inst->config->xlat_name);
+		(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
 		sql_release_socket(inst,sqlsocket);
 		return 0;
 	}
 
 	strncpy(out,row[0],ret);
 
-	DEBUG("rlm_sql: - sql_xlat finished");
+	DEBUG("rlm_sql (%s): - sql_xlat finished",
+	      inst->config->xlat_name);
 
+	(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
 	sql_release_socket(inst,sqlsocket);
 	return ret;
 }
@@ -228,14 +241,15 @@ static int sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, con
 	if (username != NULL) {
 		strNcpy(tmpuser, username, MAX_STRING_LEN);
 	} else if (strlen(inst->config->query_user)) {
-		radius_xlat(tmpuser, MAX_STRING_LEN, inst->config->query_user, request, sql_escape_func);
+		radius_xlat(tmpuser, sizeof(tmpuser), inst->config->query_user, request, sql_escape_func);
 	} else {
 		return 0;
 	}
 
 	if (*tmpuser) {
 		strNcpy(sqlusername, tmpuser, MAX_STRING_LEN * 2);
-		DEBUG2("sql_set_user:  escaped user --> '%s'", sqlusername);
+		DEBUG2("rlm_sql (%s): sql_set_user escaped user --> '%s'",
+		       inst->config->xlat_name, sqlusername);
 		vp = pairmake("SQL-User-Name", sqlusername, 0);
 		if (vp == NULL) {
 			radlog(L_ERR, "%s", librad_errstr);
@@ -248,6 +262,93 @@ static int sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, con
 	return -1;
 }
 
+/*
+ * sql groupcmp function. That way we can do group comparisons (in the users file for example)
+ * with the group memberships reciding in sql
+ * The group membership query should only return one element which is the username. The returned
+ * username will then be checked with the passed check string.
+ */
+
+static int sql_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check,
+			VALUE_PAIR *check_pairs, VALUE_PAIR **reply_pairs)
+{
+	SQLSOCK *sqlsocket;
+	SQL_ROW row;
+	SQL_INST *inst = instance;
+	char querystr[MAX_QUERY_LEN];
+	char sqlusername[2 * MAX_STRING_LEN + 10];
+
+	check_pairs = check_pairs;
+	reply_pairs = reply_pairs;
+
+	DEBUG("rlm_sql (%s): - sql_groupcmp", inst->config->xlat_name);
+	if (!check || !check->strvalue || !check->length){
+		DEBUG("rlm_sql (%s): sql_groupcmp: Illegal group name",
+		      inst->config->xlat_name);
+		return 1;
+	}
+	if (req == NULL){
+		DEBUG("rlm_sql (%s): sql_groupcmp: NULL request",
+		      inst->config->xlat_name);
+		return 1;
+	}
+	if (inst->config->groupmemb_query[0] == 0)
+		return 1;
+	/*
+	 * Set, escape, and check the user attr here
+	 */
+	if (sql_set_user(inst, req, sqlusername, 0) < 0)
+		return 1;
+	if (!radius_xlat(querystr, sizeof(querystr), inst->config->groupmemb_query, req, NULL)){
+		radlog(L_ERR, "rlm_sql (%s): xlat failed.",
+		       inst->config->xlat_name);
+		/* Remove the username we (maybe) added above */
+		pairdelete(&req->packet->vps, PW_SQL_USER_NAME);
+		return 1;
+	}
+	/* Remove the username we (maybe) added above */
+	pairdelete(&req->packet->vps, PW_SQL_USER_NAME);
+
+	sqlsocket = sql_get_socket(inst);
+	if (sqlsocket == NULL)
+		return 1;
+	if ((inst->module->sql_select_query)(sqlsocket,inst->config,querystr) <0){
+		radlog(L_ERR, "rlm_sql (%s): database query error",
+		       inst->config->xlat_name);
+		sql_release_socket(inst,sqlsocket);
+		return 1;
+	}
+	while (rlm_sql_fetch_row(sqlsocket, inst) == 0) {
+		row = sqlsocket->row;
+		if (row == NULL)
+			break;
+		if (row[0] == NULL){
+			DEBUG("rlm_sql (%s): row[0] returned NULL",
+			      inst->config->xlat_name);
+			(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
+			sql_release_socket(inst, sqlsocket);
+			return 1;
+		}
+		if (strcmp(row[0],check->strvalue) == 0){
+			DEBUG("rlm_sql (%s): - sql_groupcmp finished: User belongs in group %s",
+			      inst->config->xlat_name,
+			      (char *)check->strvalue);
+			(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
+			sql_release_socket(inst, sqlsocket);
+			return 0;
+		}
+	}
+
+	(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
+	sql_release_socket(inst,sqlsocket);
+
+	DEBUG("rlm_sql (%s): - sql_groupcmp finished: User does not belong in group %s",
+	      inst->config->xlat_name, (char *)check->strvalue);
+
+	return 1;
+}
+
+
 static int rlm_sql_instantiate(CONF_SECTION * conf, void **instance) {
 
 	SQL_INST *inst;
@@ -270,34 +371,6 @@ static int rlm_sql_instantiate(CONF_SECTION * conf, void **instance) {
 		return -1;
 	}
 
-	if (inst->config->num_sql_socks > MAX_SQL_SOCKS) {
-		radlog(L_ERR | L_CONS, "sql_instantiate:  number of sqlsockets cannot exceed MAX_SQL_SOCKS, %d", MAX_SQL_SOCKS);
-		free(inst->config);
-		free(inst);
-		return -1;
-	}
-
-	handle = lt_dlopenext(inst->config->sql_driver);
-	if (handle == NULL) {
-		radlog(L_ERR, "rlm_sql: Could not link driver %s: %s", inst->config->sql_driver, lt_dlerror());
-		radlog(L_ERR, "rlm_sql: Make sure it (and all its dependent libraries!) are in the search path of your system's ld.");
-		return -1;
-	}
-
-	inst->module = (rlm_sql_module_t *) lt_dlsym(handle, inst->config->sql_driver);
-	if (!inst->module) {
-		radlog(L_ERR, "rlm_sql: Could not link symbol %s: %s", inst->config->sql_driver, lt_dlerror());
-		return -1;
-	}
-
-	radlog(L_INFO, "rlm_sql: Driver %s loaded and linked", inst->config->sql_driver);
-	radlog(L_INFO, "rlm_sql: Attempting to connect to %s@%s:%s/%s", inst->config->sql_login, inst->config->sql_server, inst->config->sql_port, inst->config->sql_db);
-
-	if (sql_init_socketpool(inst) < 0) {
-		free(inst->config);
-		free(inst);
-		return -1;
-	}
 	xlat_name = cf_section_name2(conf);
 	if (xlat_name == NULL)
 		xlat_name = cf_section_name1(conf);
@@ -306,6 +379,47 @@ static int rlm_sql_instantiate(CONF_SECTION * conf, void **instance) {
 		xlat_register(xlat_name, sql_xlat, inst);
 	}
 
+	if (inst->config->num_sql_socks > MAX_SQL_SOCKS) {
+		radlog(L_ERR | L_CONS, "rlm_sql (%s): sql_instantiate: number of sqlsockets cannot exceed MAX_SQL_SOCKS, %d",
+		       inst->config->xlat_name, MAX_SQL_SOCKS);
+		free(inst->config);
+		free(inst);
+		return -1;
+	}
+
+	handle = lt_dlopenext(inst->config->sql_driver);
+	if (handle == NULL) {
+		radlog(L_ERR, "rlm_sql (%s): Could not link driver %s: %s",
+		       inst->config->xlat_name, inst->config->sql_driver,
+		       lt_dlerror());
+		radlog(L_ERR, "rlm_sql (%s): Make sure it (and all its dependent libraries!) are in the search path of your system's ld.",
+		       inst->config->xlat_name);
+		return -1;
+	}
+
+	inst->module = (rlm_sql_module_t *) lt_dlsym(handle, inst->config->sql_driver);
+	if (!inst->module) {
+		radlog(L_ERR, "rlm_sql (%s): Could not link symbol %s: %s",
+		       inst->config->xlat_name, inst->config->sql_driver,
+		       lt_dlerror());
+		return -1;
+	}
+
+	radlog(L_INFO, "rlm_sql (%s): Driver %s (module %s) loaded and linked",
+	       inst->config->xlat_name, inst->config->sql_driver,
+	       inst->module->name);
+	radlog(L_INFO, "rlm_sql (%s): Attempting to connect to %s@%s:%s/%s",
+	       inst->config->xlat_name, inst->config->sql_login,
+	       inst->config->sql_server, inst->config->sql_port,
+	       inst->config->sql_db);
+
+	if (sql_init_socketpool(inst) < 0) {
+		free(inst->config);
+		free(inst);
+		return -1;
+	}
+	paircompare_register(PW_SQL_GROUP, PW_USER_NAME, sql_groupcmp, inst);
+
 	*instance = inst;
 
 	return RLM_MODULE_OK;
@@ -323,6 +437,7 @@ static int rlm_sql_detach(void *instance) {
 	sql_poolfree(inst);
 	if (inst->config->xlat_name)
 		xlat_unregister(inst->config->xlat_name,sql_xlat);
+	paircompare_unregister(PW_SQL_GROUP, sql_groupcmp);
 	free(inst->config);
 	free(inst);
 
@@ -334,13 +449,11 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 
 	VALUE_PAIR *check_tmp = NULL;
 	VALUE_PAIR *reply_tmp = NULL;
-	VALUE_PAIR *passwd_item = NULL;
+	VALUE_PAIR *user_profile = NULL;
 	int     found = 0;
 	SQLSOCK *sqlsocket;
 	SQL_INST *inst = instance;
-	SQL_ROW row;
 	char    querystr[MAX_QUERY_LEN];
-	int	ret;
 
 	/* sqlusername holds the sql escaped username. The original
 	 * username is at most MAX_STRING_LEN chars long and
@@ -354,7 +467,7 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 	 */
 	if ((request->username == NULL) ||
 	    (request->username->length == 0)) {
-		radlog(L_ERR, "zero length username not permitted\n");
+		radlog(L_ERR, "rlm_sql (%s): zero length username not permitted\n", inst->config->xlat_name);
 		return RLM_MODULE_INVALID;
 	}
 
@@ -363,12 +476,13 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 	 *  After this point, ALL 'return's MUST release the SQL socket!
 	 */
 
+
 	/*
 	 * Set, escape, and check the user attr here
 	 */
 	if (sql_set_user(inst, request, sqlusername, NULL) < 0)
 		return RLM_MODULE_FAIL;
-	radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_check_query, request, sql_escape_func);
+	radius_xlat(querystr, sizeof(querystr), inst->config->authorize_check_query, request, sql_escape_func);
 
 	sqlsocket = sql_get_socket(inst);
 	if (sqlsocket == NULL) {
@@ -382,53 +496,64 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 	 *      Find the entry for the user.
 	 */
 	if (found > 0) {
-		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_check_query, request, sql_escape_func);
+		radius_xlat(querystr, sizeof(querystr), inst->config->authorize_group_check_query, request, sql_escape_func);
 		sql_getvpdata(inst, sqlsocket, &check_tmp, querystr, PW_VP_GROUPDATA);
-		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_reply_query, request, sql_escape_func);
+		radius_xlat(querystr, sizeof(querystr), inst->config->authorize_reply_query, request, sql_escape_func);
 		sql_getvpdata(inst, sqlsocket, &reply_tmp, querystr, PW_VP_USERDATA);
-		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_reply_query, request, sql_escape_func);
+		radius_xlat(querystr, sizeof(querystr), inst->config->authorize_group_reply_query, request, sql_escape_func);
 		sql_getvpdata(inst, sqlsocket, &reply_tmp, querystr, PW_VP_GROUPDATA);
 	} else if (found < 0) {
-		radlog(L_ERR, "rlm_sql:  SQL query error; rejecting user");
+		radlog(L_ERR, "rlm_sql (%s): SQL query error; rejecting user",
+		       inst->config->xlat_name);
 		sql_release_socket(inst, sqlsocket);
 		/* Remove the username we (maybe) added above */
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
 		return RLM_MODULE_FAIL;
 
 	} else {
-		int     gcheck;
-		
-		radlog(L_DBG, "rlm_sql: User %s not found", sqlusername);
+		radlog(L_DBG, "rlm_sql (%s): User %s not found in radcheck",
+		       inst->config->xlat_name, sqlusername);
 
                 /*
 		 * We didn't find the user in radcheck, so we try looking
 		 * for radgroupcheck entry
 		 */
-                radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_check_query, request, sql_escape_func);
-                gcheck = sql_getvpdata(inst, sqlsocket, &check_tmp, querystr, PW_VP_GROUPDATA);
-                radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_reply_query, request, sql_escape_func);
+                radius_xlat(querystr, sizeof(querystr), inst->config->authorize_group_check_query, request, sql_escape_func);
+                found = sql_getvpdata(inst, sqlsocket, &check_tmp, querystr, PW_VP_GROUPDATA);
+                radius_xlat(querystr, sizeof(querystr), inst->config->authorize_group_reply_query, request, sql_escape_func);
                 sql_getvpdata(inst, sqlsocket, &reply_tmp, querystr, PW_VP_GROUPDATA);
-                if (gcheck) {
-                        found = 1;
-                } else {
-                        /*
-                        * We didn't find the user, so we try looking
-                        * for a DEFAULT entry
-                        */
-                        if (sql_set_user(inst, request, sqlusername, "DEFAULT") < 0) {
-                                sql_release_socket(inst, sqlsocket);
-                                return RLM_MODULE_FAIL;
-                        }
-                        radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_check_query, request, sql_escape_func);
-                        gcheck = sql_getvpdata(inst, sqlsocket, &check_tmp, querystr, PW_VP_GROUPDATA);
-                        radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_reply_query, request, sql_escape_func);
-                        gcheck = sql_getvpdata(inst, sqlsocket, &reply_tmp, querystr, PW_VP_GROUPDATA);
-                        if (gcheck)
-                                found = 1;
-                }
         }
+	if (!found)
+		radlog(L_DBG, "rlm_sql (%s): User %s not found in radgroupcheck",
+		       inst->config->xlat_name, sqlusername);
+	if (found || (!found && inst->config->query_on_not_found)){
+		/*
+	 	* Check for a default_profile or for a User-Profile.
+		*/
+		user_profile = pairfind(request->config_items, PW_USER_PROFILE);
+		if (inst->config->default_profile[0] != 0 || user_profile != NULL){
+			char *profile = inst->config->default_profile;
+
+			if (user_profile != NULL)
+				profile = user_profile->strvalue;
+			if (profile && strlen(profile)){
+				radlog(L_DBG, "rlm_sql (%s): Checking profile %s",
+				       inst->config->xlat_name, profile);
+				if (sql_set_user(inst, request, sqlusername, profile) < 0) {
+					return RLM_MODULE_FAIL;
+				}
+				radius_xlat(querystr, sizeof(querystr), inst->config->authorize_group_check_query,
+									request, sql_escape_func);
+				found = sql_getvpdata(inst, sqlsocket, &check_tmp, querystr, PW_VP_GROUPDATA);
+				radius_xlat(querystr, sizeof(querystr), inst->config->authorize_group_reply_query,
+									request, sql_escape_func);
+				sql_getvpdata(inst, sqlsocket, &reply_tmp, querystr, PW_VP_GROUPDATA);
+			}
+		}
+	}
 	if (!found) {
-		radlog(L_DBG, "rlm_sql: DEFAULT not found");
+		radlog(L_DBG, "rlm_sql (%s): User not found",
+		       inst->config->xlat_name);
 		sql_release_socket(inst, sqlsocket);
 		/* Remove the username we (maybe) added above */
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
@@ -440,7 +565,7 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 	 * Recompile, and run 'radiusd -X'
 	 */
 
-	/* 
+	/*
 	DEBUG2("rlm_sql:  check items");
 	vp_listdebug(check_tmp);
 	DEBUG2("rlm_sql:  reply items");
@@ -448,7 +573,8 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 	*/
 
 	if (paircmp(request, request->packet->vps, check_tmp, &reply_tmp) != 0) {
-		radlog(L_INFO, "rlm_sql: Pairs do not match [%s]", sqlusername);
+		radlog(L_INFO, "rlm_sql (%s): Pairs do not match for user [%s]",
+		       inst->config->xlat_name, sqlusername);
 		/* Remove the username we (maybe) added above */
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
 		sql_release_socket(inst, sqlsocket);
@@ -462,46 +588,10 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 	pairfree(&reply_tmp);
 	pairfree(&check_tmp);
 
-	if (inst->config->authenticate_query){
-		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authenticate_query, request, sql_escape_func);
-	
-		/* Remove the username we (maybe) added above */
-		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
-
-		if (rlm_sql_select_query(sqlsocket, inst, querystr)) {
-			radlog(L_ERR, "rlm_sql_authorize: database query error");
-			sql_release_socket(inst, sqlsocket);
-			return RLM_MODULE_FAIL;
-		}
-		ret = rlm_sql_fetch_row(sqlsocket, inst);
-
-		(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
-		sql_release_socket(inst, sqlsocket);
-
-		if (ret) {
-			radlog(L_ERR, "rlm_sql_authorize: query failed");
-			return RLM_MODULE_FAIL;
-		}
-
-		row = sqlsocket->row;
-		if (row == NULL) {
-			radlog(L_ERR, "rlm_sql_authorize: no rows returned from query (no such user)");
-			return RLM_MODULE_OK;
-		}
-
-		if (row[0] == NULL) {
-			radlog(L_ERR, "rlm_sql_authorize: row[0] returned NULL.");
-			return RLM_MODULE_OK;
-		}
-		if ((passwd_item = pairmake("User-Password",row[0],T_OP_SET)) != NULL)
-			pairadd(&request->config_items,passwd_item);
-		goto move_on;
-	}
 	/* Remove the username we (maybe) added above */
 	pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
 	sql_release_socket(inst, sqlsocket);
 
-move_on:
 	return RLM_MODULE_OK;
 }
 
@@ -513,6 +603,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 	SQLSOCK *sqlsocket = NULL;
 	VALUE_PAIR *pair;
 	SQL_INST *inst = instance;
+	int	ret = RLM_MODULE_OK;
 	int     numaffected = 0;
 	int     acctstatustype = 0;
 	char    querystr[MAX_QUERY_LEN];
@@ -531,7 +622,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 	if ((pair = pairfind(request->packet->vps, PW_ACCT_STATUS_TYPE)) != NULL) {
 		acctstatustype = pair->lvalue;
 	} else {
-		radius_xlat(logstr, MAX_QUERY_LEN, "rlm_sql:  packet has no account status type.  [user '%{User-Name}', nas '%{NAS-IP-Address}']", request, sql_escape_func);
+		radius_xlat(logstr, sizeof(logstr), "rlm_sql: packet has no account status type.  [user '%{User-Name}', nas '%{NAS-IP-Address}']", request, sql_escape_func);
 		radlog(L_ERR, logstr);
 		return RLM_MODULE_INVALID;
 	}
@@ -543,16 +634,20 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			 */
 		case PW_STATUS_ACCOUNTING_ON:
 		case PW_STATUS_ACCOUNTING_OFF:
-			radlog(L_INFO, "rlm_sql:  received Acct On/Off packet");
-			radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_onoff_query, request, sql_escape_func);
+			radlog(L_INFO, "rlm_sql (%s): received Acct On/Off packet", inst->config->xlat_name);
+			radius_xlat(querystr, sizeof(querystr), inst->config->accounting_onoff_query, request, sql_escape_func);
 			query_log(inst, querystr);
 
 			sqlsocket = sql_get_socket(inst);
 			if (sqlsocket == NULL)
 				return(RLM_MODULE_FAIL);
 			if (querystr) {
-				if (rlm_sql_query(sqlsocket, inst, querystr))
-					radlog(L_ERR, "rlm_sql: Couldn't update SQL accounting for Acct On/Off packet - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
+				if (rlm_sql_query(sqlsocket, inst, querystr)) {
+					radlog(L_ERR, "rlm_sql (%s): Couldn't update SQL accounting for Acct On/Off packet - %s",
+					       inst->config->xlat_name,
+					       (char *)(inst->module->sql_error)(sqlsocket, inst->config));
+					ret = RLM_MODULE_FAIL;
+				}
 				(inst->module->sql_finish_query)(sqlsocket, inst->config);
 			}
 
@@ -568,15 +663,19 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			 */
 			sql_set_user(inst, request, sqlusername, NULL);
 
-			radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_update_query, request, sql_escape_func);
+			radius_xlat(querystr, sizeof(querystr), inst->config->accounting_update_query, request, sql_escape_func);
 			query_log(inst, querystr);
 
 			sqlsocket = sql_get_socket(inst);
 			if (sqlsocket == NULL)
 				return(RLM_MODULE_FAIL);
 			if (querystr) {
-				if (rlm_sql_query(sqlsocket, inst, querystr))
-					radlog(L_ERR, "rlm_sql: Couldn't update SQL accounting for ALIVE packet - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
+				if (rlm_sql_query(sqlsocket, inst, querystr)) {
+					radlog(L_ERR, "rlm_sql (%s): Couldn't update SQL accounting for ALIVE packet - %s",
+					       inst->config->xlat_name,
+					       (char *)(inst->module->sql_error)(sqlsocket, inst->config));
+					ret = RLM_MODULE_FAIL;
+				}
 				(inst->module->sql_finish_query)(sqlsocket, inst->config);
 			}
 
@@ -592,7 +691,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			 */
 			sql_set_user(inst, request, sqlusername, NULL);
 
-			radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_start_query, request, sql_escape_func);
+			radius_xlat(querystr, sizeof(querystr), inst->config->accounting_start_query, request, sql_escape_func);
 			query_log(inst, querystr);
 
 			sqlsocket = sql_get_socket(inst);
@@ -600,19 +699,24 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 				return(RLM_MODULE_FAIL);
 			if (querystr) {
 				if (rlm_sql_query(sqlsocket, inst, querystr)) {
-					radlog(L_ERR, "rlm_sql: Couldn't update SQL accounting" " for START packet - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
+					radlog(L_ERR, "rlm_sql (%s): Couldn't update SQL accounting" " for START packet - %s",
+					       inst->config->xlat_name,
+					       (char *)(inst->module->sql_error)(sqlsocket, inst->config));
 
 					/*
 					 * We failed the insert above.  It's probably because 
 					 * the stop record came before the start.  We try an
 					 * our alternate query now (typically an UPDATE)
 					 */
-					radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_start_query_alt, request, sql_escape_func);
+					radius_xlat(querystr, sizeof(querystr), inst->config->accounting_start_query_alt, request, sql_escape_func);
 					query_log(inst, querystr);
 
 					if (querystr) {
 						if (rlm_sql_query(sqlsocket, inst, querystr)) {
-							radlog(L_ERR, "rlm_sql: Couldn't update SQL" "accounting START record - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
+							radlog(L_ERR, "rlm_sql (%s): Couldn't update SQL" "accounting START record - %s",
+							       inst->config->xlat_name, 
+							       (char *)(inst->module->sql_error)(sqlsocket, inst->config));
+							ret = RLM_MODULE_FAIL;
 						}
 						(inst->module->sql_finish_query)(sqlsocket, inst->config);
 					}
@@ -631,7 +735,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			 */
 			sql_set_user(inst, request, sqlusername, NULL);
 
-			radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_stop_query, request, sql_escape_func);
+			radius_xlat(querystr, sizeof(querystr), inst->config->accounting_stop_query, request, sql_escape_func);
 			query_log(inst, querystr);
 
 			sqlsocket = sql_get_socket(inst);
@@ -639,7 +743,10 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 				return(RLM_MODULE_FAIL);
 			if (querystr) {
 				if (rlm_sql_query(sqlsocket, inst, querystr)) {
-					radlog(L_ERR, "rlm_sql: Couldn't update SQL accounting STOP record - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
+					radlog(L_ERR, "rlm_sql (%s): Couldn't update SQL accounting STOP record - %s",
+					       inst->config->xlat_name,
+					       (char *)(inst->module->sql_error)(sqlsocket, inst->config));
+					ret = RLM_MODULE_FAIL;
 				}
 				else {
 					numaffected = (inst->module->sql_affected_rows)(sqlsocket, inst->config);
@@ -661,19 +768,20 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 					                acctsessiontime = pair->lvalue;
 	
 					        if (acctsessiontime <= 0) {
-				        	        radius_xlat(logstr, MAX_QUERY_LEN, "rlm_sql:  Stop packet with zero session length.  (user '%{User-Name}', nas '%{NAS-IP-Address}')", request, sql_escape_func);
+				        	        radius_xlat(logstr, sizeof(logstr), "rlm_sql: Stop packet with zero session length.  (user '%{User-Name}', nas '%{NAS-IP-Address}')", request, sql_escape_func);
 					                radlog(L_ERR, logstr);
 				        	        sql_release_socket(inst, sqlsocket);
 				                	return RLM_MODULE_NOOP;
 					        }
 #endif
 
-						radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_stop_query_alt, request, sql_escape_func);
+						radius_xlat(querystr, sizeof(querystr), inst->config->accounting_stop_query_alt, request, sql_escape_func);
 						query_log(inst, querystr);
 
 						if (querystr) {
 							if (rlm_sql_query(sqlsocket, inst, querystr)) {
 								radlog(L_ERR, "rlm_sql: Couldn't insert SQL accounting STOP record - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
+								ret = RLM_MODULE_FAIL;
 							}
 							(inst->module->sql_finish_query)(sqlsocket, inst->config);
 						}
@@ -682,11 +790,20 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 				(inst->module->sql_finish_query)(sqlsocket, inst->config);
 			}
 			break;
+
+			/*
+			 *	Anything else is ignored.
+			 */
+		default:
+			radlog(L_INFO, "rlm_sql (%s): Unsupported Acct-Status-Type = %d", inst->config->xlat_name, acctstatustype);
+			return RLM_MODULE_NOOP;
+			break;
+
 	}
 
 	sql_release_socket(inst, sqlsocket);
 
-	return RLM_MODULE_OK;
+	return ret;
 }
 
 
@@ -710,6 +827,8 @@ static int rlm_sql_checksimul(void *instance, REQUEST * request) {
         char            *call_num = NULL;
 	VALUE_PAIR      *vp;
 	int		ret;
+	uint32_t	nas_addr = 0;
+	int		nas_port = 0;
 
 	/* If simul_count_query is not defined, we don't do any checking */
 	if (inst->config->simul_count_query[0] == 0) {
@@ -717,43 +836,44 @@ static int rlm_sql_checksimul(void *instance, REQUEST * request) {
 	}
 
 	if((request->username == NULL) || (request->username->length == 0)) {
-		radlog(L_ERR, "Zero Length username not permitted\n");
+		radlog(L_ERR, "rlm_sql (%s): Zero Length username not permitted\n", inst->config->xlat_name);
 		return RLM_MODULE_INVALID;
 	}
 
+
+	if(sql_set_user(inst, request, sqlusername, 0) <0)
+		return RLM_MODULE_FAIL;
+
+	radius_xlat(querystr, sizeof(querystr), inst->config->simul_count_query, request, NULL);
+
 	/* initialize the sql socket */
 	sqlsocket = sql_get_socket(inst);
 	if(sqlsocket == NULL)
 		return RLM_MODULE_FAIL;
 
-	if(sql_set_user(inst, request, sqlusername, 0) <0)
-		return RLM_MODULE_FAIL;
-
-	radius_xlat(querystr, MAX_QUERY_LEN, inst->config->simul_count_query, request, NULL);
 	if(rlm_sql_select_query(sqlsocket, inst, querystr)) {
-		radlog(L_ERR, "sql_checksimul: Database query failed");
+		radlog(L_ERR, "rlm_sql (%s) sql_checksimul: Database query failed", inst->config->xlat_name);
 		sql_release_socket(inst, sqlsocket);
 		return RLM_MODULE_FAIL;
 	}
 
 	ret = rlm_sql_fetch_row(sqlsocket, inst);
 
-	if (ret == 0) {
-		row = sqlsocket->row;
-	}
-	(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
-
-	if (ret) {
+	if (ret != 0) {
+		(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
 		sql_release_socket(inst, sqlsocket);
 		return RLM_MODULE_FAIL;
 	}
 
+	row = sqlsocket->row;
 	if (row == NULL) {
+		(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
 		sql_release_socket(inst, sqlsocket);
 		return RLM_MODULE_FAIL;
 	}
 
 	request->simul_count = atoi(row[0]);
+	(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
 
 	if(request->simul_count < request->simul_max) {
 		sql_release_socket(inst, sqlsocket);
@@ -764,12 +884,13 @@ static int rlm_sql_checksimul(void *instance, REQUEST * request) {
 
 	if (inst->config->simul_verify_query[0] == 0) {
 		/* No verify query defined, so skip verify step and rely on count query only */
+		sql_release_socket(inst, sqlsocket);
 		return RLM_MODULE_OK;
 	}
 
-	radius_xlat(querystr, MAX_QUERY_LEN, inst->config->simul_verify_query, request, NULL);
+	radius_xlat(querystr, sizeof(querystr), inst->config->simul_verify_query, request, NULL);
 	if(rlm_sql_select_query(sqlsocket, inst, querystr)) {
-		radlog(L_ERR, "sql_checksimul: Database query error");
+		radlog(L_ERR, "rlm_sql (%s): sql_checksimul: Database query error", inst->config->xlat_name);
 		sql_release_socket(inst, sqlsocket);
 		return RLM_MODULE_FAIL;
 	}
@@ -789,13 +910,34 @@ static int rlm_sql_checksimul(void *instance, REQUEST * request) {
 		row = sqlsocket->row;
 		if (row == NULL)
 			break;
-		check = rad_check_ts((uint32_t)row[3], (int)row[4], row[2], row[1]);
+		if (!row[2]){
+			(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
+			sql_release_socket(inst, sqlsocket);
+			DEBUG("rlm_sql (%s): Cannot zap stale entry. No username present in entry.", inst->config->xlat_name);
+			return RLM_MODULE_FAIL;
+		}
+		if (!row[1]){
+			(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
+			sql_release_socket(inst, sqlsocket);
+			DEBUG("rlm_sql (%s): Cannot zap stale entry. No session id in entry.", inst->config->xlat_name);
+			return RLM_MODULE_FAIL;
+		}
+		if (row[3])
+			nas_addr = inet_addr(row[3]);
+		if (row[4])
+			nas_port = atoi(row[4]);
+
+		check = rad_check_ts(nas_addr, nas_port, row[2], row[1]);
 
                 /*
                  *      Failed to check the terminal server for
                  *      duplicate logins: Return an error.
                  */
 		if (check < 0) {
+			(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
+			sql_release_socket(inst, sqlsocket);
+			DEBUG("rlm_sql (%s) rad_check_ts() failed.",
+			      inst->config->xlat_name);
 			return RLM_MODULE_FAIL;
 		}
 
@@ -815,19 +957,18 @@ static int rlm_sql_checksimul(void *instance, REQUEST * request) {
                         /*
                          *      Stale record - zap it.
                          */
-			if(inst->config->simul_zap_query[0] != 0) {
-				SQLSOCK *sqlsocket1;
-				radlog(L_ERR, "rlm_sql: Deleting stale session [%s] (from %s/%s)", 
-								row[1],row[3],row[4]);
-				sqlsocket1 = sql_get_socket(inst);
-				sprintf(querystr, inst->config->simul_zap_query, row[0]);
-				if (rlm_sql_query(sqlsocket1, inst, querystr)) {
-					radlog(L_ERR, "rlm_sql: Deletion of stale session [%s] failed",
-								row[1]);
-				}
-				(inst->module->sql_finish_select_query)(sqlsocket1, inst->config);
-				sql_release_socket(inst, sqlsocket1);
-			}
+			uint32_t framed_addr = 0;
+			char proto = 'P';
+
+			if (row[5])
+				framed_addr = inet_addr(row[5]);
+			if (row[7])
+				if (strcmp(row[7],"SLIP") == 0)
+					proto = 'S';
+
+			session_zap(request->packet->sockfd,
+			nas_addr,nas_port,row[2],row[1],
+			framed_addr, proto,0);
 		}
 	}
 
@@ -851,7 +992,10 @@ module_t rlm_sql = {
 		rlm_sql_authorize,	/* authorization */
 		NULL,			/* preaccounting */
 		rlm_sql_accounting,	/* accounting */
-		rlm_sql_checksimul	/* checksimul */
+		rlm_sql_checksimul,	/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		NULL			/* post-auth */
 	},
 	rlm_sql_detach,		/* detach */
 	rlm_sql_destroy,	/* destroy */
