@@ -2,7 +2,7 @@
  *  sql.c		rlm_sql - FreeRADIUS SQL Module
  *		Main code directly taken from ICRADIUS
  *
- * Version:	$Id: sql.c,v 1.54 2002/07/11 18:25:35 cparker Exp $
+ * Version:	$Id: sql.c,v 1.58 2002/11/08 20:45:10 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -54,20 +54,25 @@
 
 
 /*
- * Connect to a server.  If error, set this socket's state to be "sockunconnected"
- * and set a grace period, during which we won't try connecting again (to prevent unduly
- * lagging the server and being impolite to a DB server that may be having other 
- * issues).  If successful in connecting, set state to sockconnected.   - chad
+ * Connect to a server.  If error, set this socket's state to be
+ * "sockunconnected" and set a grace period, during which we won't try
+ * connecting again (to prevent unduly lagging the server and being
+ * impolite to a DB server that may be having other issues).  If
+ * successful in connecting, set state to sockconnected.
+ * - chad
  */
 static int connect_single_socket(SQLSOCK *sqlsocket, SQL_INST *inst) {
-	radlog(L_DBG, "rlm_sql:  Attempting to connect #%d", sqlsocket->id);
+	radlog(L_DBG, "rlm_sql (%s): Attempting to connect %s #%d",
+	       inst->config->xlat_name, inst->module->name, sqlsocket->id);
+
 	if ((inst->module->sql_init_socket)(sqlsocket, inst->config) < 0) {
-		radlog(L_CONS | L_ERR, "rlm_sql:  Failed to connect DB handle #%d", sqlsocket->id);
+		radlog(L_CONS | L_ERR, "rlm_sql (%s): Failed to connect DB handle #%d", inst->config->xlat_name, sqlsocket->id);
 		inst->connect_after = time(NULL) + inst->config->connect_failure_retry_delay;
 		sqlsocket->state = sockunconnected;
 		return(-1);
 	} else {
-		radlog(L_DBG, "rlm_sql:  Connected new DB handle, #%d", sqlsocket->id);
+		radlog(L_DBG, "rlm_sql (%s): Connected new DB handle, #%d",
+		       inst->config->xlat_name, sqlsocket->id);
 		sqlsocket->state = sockconnected;
 		return(0);
 	}
@@ -92,7 +97,8 @@ int sql_init_socketpool(SQL_INST * inst) {
 	inst->socknr = 0;
 
 	for (i = 0; i < inst->config->num_sql_socks; i++) {
-		radlog(L_DBG, "rlm_sql: starting %d", i);
+		radlog(L_DBG, "rlm_sql (%s): starting %d",
+		       inst->config->xlat_name, i);
 
 		sqlsocket = rad_malloc(sizeof(SQLSOCK));
 		if (sqlsocket == NULL) {
@@ -159,7 +165,8 @@ void sql_poolfree(SQL_INST * inst) {
  *************************************************************************/
 int sql_close_socket(SQL_INST *inst, SQLSOCK * sqlsocket) {
 
-	radlog(L_DBG, "rlm_sql: Closing sqlsocket %d", sqlsocket->id);
+	radlog(L_DBG, "rlm_sql (%s): Closing sqlsocket %d",
+	       inst->config->xlat_name, sqlsocket->id);
 	(inst->module->sql_close)(sqlsocket, inst->config);
 #if HAVE_SEMAPHORE_H
 	sem_destroy(sqlsocket->semaphore);
@@ -181,7 +188,7 @@ SQLSOCK * sql_get_socket(SQL_INST * inst) {
 	int tried_to_connect = 0;
 
 	while (inst->used == inst->config->num_sql_socks) {
-		radlog(L_ERR, "rlm_sql: All sockets are being used! Please increase the maximum number of sockets!");
+		radlog(L_ERR, "rlm_sql (%s): All sockets are being used! Please increase the maximum number of sockets!", inst->config->xlat_name);
 		return NULL;
 	}
 
@@ -213,13 +220,13 @@ SQLSOCK * sql_get_socket(SQL_INST * inst) {
 		 */
 		if ((cur->state == sockunconnected) && (time(NULL) > inst->connect_after)) {
 			tried_to_connect = 1;
-			radlog(L_INFO, "rlm_sql: Trying to (re)connect an unconnected handle...");
+			radlog(L_INFO, "rlm_sql (%s): Trying to (re)connect an unconnected handle...", inst->config->xlat_name);
 			connect_single_socket(cur, inst);
 		}
 
 		/* if we still aren't connected, ignore this handle */
 		if (cur->state == sockunconnected) {
-			radlog(L_DBG, "rlm_sql: Ignoring unconnected handle");
+			radlog(L_DBG, "rlm_sql (%s): Ignoring unconnected handle", inst->config->xlat_name);
 			continue;
 		}
 
@@ -232,13 +239,14 @@ SQLSOCK * sql_get_socket(SQL_INST * inst) {
 #ifndef HAVE_SEMAPHORE_H
 			cur->in_use = SQLSOCK_LOCKED;
 #endif
-			radlog(L_DBG, "rlm_sql: Reserving sql socket id: %d", cur->id);
+			radlog(L_DBG, "rlm_sql (%s): Reserving sql socket id: %d",
+			       inst->config->xlat_name, cur->id);
 			return cur;
 		}
 	}
 
 	/* We get here if every DB handle is unconnected and unconnectABLE */
-	radlog((tried_to_connect = 0) ? (L_DBG) : (L_CONS | L_ERR), "rlm_sql:  There are no DB handles to use!");
+	radlog((tried_to_connect == 0) ? (L_DBG) : (L_CONS | L_ERR), "rlm_sql (%s): There are no DB handles to use!", inst->config->xlat_name);
 	return NULL;
 }
 
@@ -258,7 +266,8 @@ int sql_release_socket(SQL_INST * inst, SQLSOCK * sqlsocket) {
 	sqlsocket->in_use = SQLSOCK_UNLOCKED;
 #endif
 
-	radlog(L_DBG, "rlm_sql: Released sql socket id: %d", sqlsocket->id);
+	radlog(L_DBG, "rlm_sql (%s): Released sql socket id: %d",
+	       inst->config->xlat_name, sqlsocket->id);
 
 	return 0;
 }
@@ -280,13 +289,21 @@ int sql_userparse(VALUE_PAIR ** first_pair, SQL_ROW row, int querymode) {
 	int pairmode = T_EOL;
 
 	if ((attr = dict_attrbyname(row[2])) == (DICT_ATTR *) NULL) {
-		radlog(L_ERR | L_CONS, "rlm_sql: unknown attribute %s", row[2]);
+		radlog(L_ERR | L_CONS, "rlm_sql: unknown attribute %s",
+		       row[2]);
 		return (-1);
 	}
 
 	if (row[4] != NULL && strlen(row[4]) > 0) {
 		ptr = row[4];
 		pairmode = gettoken(&ptr, buf, sizeof(buf));
+	} else {
+		/*
+		 *  'op' fields of NULL are a plague, and a bane on the
+		 *  existence of mankind.
+		 */
+		radlog(L_ERR, "rlm_sql: The 'op' field for attribute '%s = %s' is NULL, or non-existent.", row[2], row[3]);
+		radlog(L_ERR, "rlm_sql: You MUST FIX THIS if you want the configuration to behave as you expect.");
 	}
 	if (pairmode <= T_EOL) pairmode = T_OP_CMP_EQ;
 
@@ -322,14 +339,15 @@ int rlm_sql_fetch_row(SQLSOCK *sqlsocket, SQL_INST *inst) {
 
 	if (ret == SQL_DOWN) {
 		if (connect_single_socket(sqlsocket, inst) < 0) {
-			radlog(L_ERR, "rlm_sql: reconnect failed, database down?");
+			radlog(L_ERR, "rlm_sql (%s): reconnect failed, database down?", inst->config->xlat_name);
 			return -1;
 		}
 
 		ret = (inst->module->sql_fetch_row)(sqlsocket, inst->config);
 
 		if (ret) {
-			radlog(L_ERR, "rlm_sql: failed after re-connect");
+			radlog(L_ERR, "rlm_sql (%s): failed after re-connect",
+			       inst->config->xlat_name);
 			return -1;
 		}
 	}
@@ -351,14 +369,15 @@ int rlm_sql_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query) {
 
 	if (ret == SQL_DOWN) {
 		if (connect_single_socket(sqlsocket, inst) < 0) {
-			radlog(L_ERR, "rlm_sql: reconnect failed, database down?");
+			radlog(L_ERR, "rlm_sql (%s): reconnect failed, database down?", inst->config->xlat_name);
 			return -1;
 		}
 
 		ret = (inst->module->sql_query)(sqlsocket, inst->config, query);
 
 		if (ret) {
-			radlog(L_ERR, "rlm_sql: failed after re-connect");
+			radlog(L_ERR, "rlm_sql (%s): failed after re-connect",
+			       inst->config->xlat_name);
 			return -1;
 		}
 	}
@@ -380,14 +399,15 @@ int rlm_sql_select_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query) {
 
 	if (ret == SQL_DOWN) {
 		if (connect_single_socket(sqlsocket, inst) < 0) {
-			radlog(L_ERR, "rlm_sql: reconnect failed, database down?");
+			radlog(L_ERR, "rlm_sql (%s): reconnect failed, database down?", inst->config->xlat_name);
 			return -1;
 		}
 
 		ret = (inst->module->sql_select_query)(sqlsocket, inst->config, query);
 
 		if (ret) {
-			radlog(L_ERR, "rlm_sql: failed after re-connect");
+			radlog(L_ERR, "rlm_sql (%s): failed after re-connect",
+			       inst->config->xlat_name);
 			return -1;
 		}
 	}
@@ -417,7 +437,7 @@ int sql_getvpdata(SQL_INST * inst, SQLSOCK * sqlsocket, VALUE_PAIR **pair, char
 		if (!row)
 			break;
 		if (sql_userparse(pair, row, mode) != 0) {
-			radlog(L_ERR | L_CONS, "rlm_sql:  Error getting data from database");
+			radlog(L_ERR | L_CONS, "rlm_sql (%s): Error getting data from database", inst->config->xlat_name);
 			(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
 			return -1;
 		}
@@ -428,203 +448,14 @@ int sql_getvpdata(SQL_INST * inst, SQLSOCK * sqlsocket, VALUE_PAIR **pair, char
 	return rows;
 }
 
-
-static int got_alrm;
-static void
-alrm_handler(int i) {
-	got_alrm = 1;
-}
-
-/*************************************************************************
- *
- *	Function: sql_check_ts
- *
- *	Purpose: Checks the terminal server for a spacific login entry
- *
- *************************************************************************/
-static int sql_check_ts(SQL_ROW row) {
-
-	int     pid, st, e;
-	int     n;
-	NAS    *nas;
-	char    session_id[12];
-	char   *s;
-	void    (*handler) (int);
-
-	/*
-	 *      Find NAS type.
-	 */
-	if ((nas = nas_find(ip_addr(row[4]))) == NULL) {
-		radlog(L_ERR, "rlm_sql:  unknown NAS [%s]", row[4]);
-		return -1;
-	}
-
-	/*
-	 *      Fork.
-	 */
-	handler = signal(SIGCHLD, SIG_DFL);
-	if ((pid = fork()) < 0) {
-		radlog(L_ERR, "rlm_sql: fork: %s", strerror(errno));
-		signal(SIGCHLD, handler);
-		return -1;
-	}
-
-	if (pid > 0) {
-		/*
-		 *      Parent - Wait for checkrad to terminate.
-		 *      We timeout in 10 seconds.
-		 */
-		got_alrm = 0;
-		signal(SIGALRM, alrm_handler);
-		alarm(10);
-		while ((e = waitpid(pid, &st, 0)) != pid)
-			if (e < 0 && (errno != EINTR || got_alrm))
-				break;
-		alarm(0);
-		signal(SIGCHLD, handler);
-		if (got_alrm) {
-			kill(pid, SIGTERM);
-			sleep(1);
-			kill(pid, SIGKILL);
-			radlog(L_ERR, "rlm_sql:  Check-TS: timeout waiting for checkrad");
-			return 2;
-		}
-		if (e < 0) {
-			radlog(L_ERR, "rlm_sql:  Check-TS: unknown error in waitpid()");
-			return 2;
-		}
-		return WEXITSTATUS(st);
-	}
-
-	/*
-	 *      Child - exec checklogin with the right parameters.
-	 */
-	for (n = 32; n >= 3; n--)
-		close(n);
-
-	sprintf(session_id, "%.8s", row[1]);
-
-	s = CHECKRAD2;
-	execl(CHECKRAD2, "checkrad", nas->nastype, row[4], row[5],
-			row[2], session_id, NULL);
-	if (errno == ENOENT) {
-		s = CHECKRAD1;
-		execl(CHECKRAD1, "checklogin", nas->nastype, row[4], row[5],
-				row[2], session_id, NULL);
-	}
-	radlog(L_ERR, "rlm_sql:  Check-TS: exec %s: %s", s, strerror(errno));
-
-	/*
-	 *      Exit - 2 means "some error occured".
-	 */
-	exit(2);
-	return -1;
-}
-
-
-/*************************************************************************
- *
- *	Function: sql_check_multi
- *
- *	Purpose: Check radius accounting for duplicate logins
- *
- *************************************************************************/
-int sql_check_multi(SQL_INST * inst, SQLSOCK * sqlsocket, char *name, VALUE_PAIR * request, int maxsimul) {
-
-	char    querystr[MAX_QUERY_LEN];
-	char    authstr[256];
-	VALUE_PAIR *fra;
-	SQL_ROW row;
-	int     count = 0;
-	uint32_t ipno = 0;
-	int     mpp = 1;
-
-	sprintf(authstr, "UserName = '%s'", name);
-	sprintf(querystr, "SELECT COUNT(*) FROM %s WHERE %s AND AcctStopTime = 0", inst->config->sql_acct_table, authstr);
-	
-
-	if (rlm_sql_select_query(sqlsocket, inst, querystr)) {
-		radlog(L_ERR, "sql_check_multi: database query error");
-		return -1;
-	}
-
-	if (rlm_sql_fetch_row(sqlsocket, inst->config)) {
-		(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
-		return -1;
-	}
-
-	row = sqlsocket->row;
-	if (row != NULL) {
-		count = atoi(row[0]);
-	} else {
-		count = 0;
-	}
-	(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
-
-	if (count < maxsimul)
-		return 0;
-
-	/*
-	 * *      Setup some stuff, like for MPP detection.
-	 */
-	if ((fra = pairfind(request, PW_FRAMED_IP_ADDRESS)) != NULL)
-		ipno = htonl(fra->lvalue);
-
-	count = 0;
-	sprintf(querystr, "SELECT * FROM %s WHERE %s AND AcctStopTime = 0", inst->config->sql_acct_table, authstr);
-
-	if (rlm_sql_select_query(sqlsocket, inst, querystr)) {
-		radlog(L_ERR, "sql_check_multi: database query error");
-		return -1;
-	}
-	while (rlm_sql_fetch_row(sqlsocket, inst) == 0) {
-		int     check;
-		row = sqlsocket->row;
-		if (row == NULL) {
-			break;
-		}
-		check = sql_check_ts(row);
-
-		if (check == 1) {
-			count++;
-
-			if (ipno && atoi(row[19]) == ipno)
-				mpp = 2;
-
-		} else if (check == 2)
-			radlog(L_ERR, "rlm_sql:  Problem with checkrad [%s] (from nas %s)", name, row[4]);
-		else {
-			/*
-			 *      False record - zap it
-			 */
-
-			if (inst->config->deletestalesessions) {
-				SQLSOCK *sqlsocket1;
-
-				radlog(L_ERR, "rlm_sql:  Deleteing stale session [%s] (from nas %s/%s)", row[2], row[4], row[5]);
-				sqlsocket1 = sql_get_socket(inst);
-				sprintf(querystr, "DELETE FROM %s WHERE RadAcctId = '%s'", inst->config->sql_acct_table, row[0]);
-				if(rlm_sql_query(sqlsocket1, inst, querystr)) {
-					radlog(L_ERR, "rlm_sql: database query error");
-				} else {
-					(inst->module->sql_finish_query)(sqlsocket1, inst->config);
-					sql_release_socket(inst, sqlsocket1);
-				}
-			}
-		}
-	}
-	(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
-
-	return (count < maxsimul) ? 0 : mpp;
-}
-
 void query_log(SQL_INST * inst, char *querystr) {
 	FILE   *sqlfile = NULL;
 
 	if (inst->config->sqltrace) {
 		if ((sqlfile = fopen(inst->config->tracefile, "a")) == (FILE *) NULL) {
-			radlog(L_ERR, "rlm_sql: Couldn't open file %s",
-					inst->config->tracefile);
+			radlog(L_ERR, "rlm_sql (%s): Couldn't open file %s",
+			       inst->config->xlat_name,
+			       inst->config->tracefile);
 		} else {
 			int fd = fileno(sqlfile);
 			
