@@ -3,7 +3,7 @@
  *		accounting:     Functions to write radwtmp file.
  *		Also contains handler for "Group".
  *
- * Version:	$Id: rlm_unix.c,v 1.46 2002/05/06 14:29:41 aland Exp $
+ * Version:	$Id: rlm_unix.c,v 1.50 2002/10/28 20:14:03 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
-static const char rcsid[] = "$Id: rlm_unix.c,v 1.46 2002/05/06 14:29:41 aland Exp $";
+static const char rcsid[] = "$Id: rlm_unix.c,v 1.50 2002/10/28 20:14:03 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -84,7 +84,7 @@ static CONF_PARSER module_config[] = {
 	 *	Cache the password by default.
 	 */
 	{ "cache",    PW_TYPE_BOOLEAN,
-	  offsetof(struct unix_instance,cache_passwd), NULL, "yes" },
+	  offsetof(struct unix_instance,cache_passwd), NULL, "no" },
 	{ "passwd",   PW_TYPE_STRING_PTR,
 	  offsetof(struct unix_instance,passwd_file), NULL,  NULL },
 	{ "shadow",   PW_TYPE_STRING_PTR,
@@ -92,7 +92,7 @@ static CONF_PARSER module_config[] = {
 	{ "group",    PW_TYPE_STRING_PTR,
 	  offsetof(struct unix_instance,group_file), NULL,   NULL },
 	{ "radwtmp",  PW_TYPE_STRING_PTR,
-	  offsetof(struct unix_instance,radwtmp), NULL,   "${logdir}/radwtmp" },
+	  offsetof(struct unix_instance,radwtmp), NULL,   "NULL" },
 	{ "usegroup", PW_TYPE_BOOLEAN,
 	  offsetof(struct unix_instance,usegroup), NULL,     "no" },
 	{ "cache_reload", PW_TYPE_INTEGER,
@@ -226,7 +226,7 @@ static int groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_PAI
 		return -1;
 
 	if (group_inst->group_file)
-		grp = fgetgrnam(group_inst->passwd_file, (char *)check->strvalue);
+		grp = fgetgrnam(group_inst->group_file, (char *)check->strvalue);
 	else
 		grp = getgrnam((char *)check->strvalue);
 	if (grp == NULL)
@@ -287,7 +287,7 @@ static int unix_instantiate(CONF_SECTION *conf, void **instance)
 			free((char *) inst->passwd_file);
 			free((char *) inst->shadow_file);
 			free((char *) inst->group_file);
-			free((char *) inst->radwtmp);
+			if (inst->radwtmp) free((char *) inst->radwtmp);
 			free(inst);
 			return -1;
 		}
@@ -328,7 +328,7 @@ static int unix_detach(void *instance)
 	free((char *) inst->passwd_file);
 	free((char *) inst->shadow_file);
 	free((char *) inst->group_file);
-	free((char *) inst->radwtmp);
+	if (inst->radwtmp) free((char *) inst->radwtmp);
 	if (inst->cache) {
 		unix_freepwcache(inst->cache);
 	}
@@ -637,7 +637,7 @@ static char *uue(void *in)
 static int unix_accounting(void *instance, REQUEST *request)
 {
 	VALUE_PAIR	*vp;
-	NAS		*cl;
+	RADCLIENT	*cl;
 	FILE		*fp;
 	struct utmp	ut;
 	time_t		t;
@@ -653,6 +653,14 @@ static int unix_accounting(void *instance, REQUEST *request)
 	int		nas_port_type = 0;
 	struct unix_instance *inst = (struct unix_instance *) instance;
 
+	/*
+	 *	No radwtmp.  Don't do anything.
+	 */
+	if (!inst->radwtmp) {
+		DEBUG2("rlm_unix: No radwtmp file configured.  Ignoring accounting request.");
+		return RLM_MODULE_NOOP;
+	}
+
 	/*
 	 *	Which type is this.
 	 */
@@ -739,7 +747,7 @@ static int unix_accounting(void *instance, REQUEST *request)
 	 *	and address so that the tty field is unique.
 	 */
 	s = "";
-	if ((cl = nas_find(nas_address)) != NULL)
+	if ((cl = client_find(nas_address)) != NULL)
 		s = cl->shortname;
 	if (s == NULL || s[0] == 0) s = uue(&(nas_address));
 	sprintf(buf, "%03d:%s", nas_port, s);
@@ -805,7 +813,10 @@ module_t rlm_unix = {
 	  NULL,                 /* authorization */
 	  NULL,                 /* preaccounting */
 	  unix_accounting,      /* accounting */
-	  NULL                  /* checksimul */
+	  NULL,                  /* checksimul */
+	  NULL,			/* pre-proxy */
+	  NULL,			/* post-proxy */
+	  NULL			/* post-auth */
   },
   unix_detach,                 	/* detach */
   unix_destroy,                  /* destroy */
